# Copyright (c) 2012 Santosh Philip
# =======================================================================
#  Distributed under the MIT License.
#  (See accompanying file LICENSE or copy at
#  http://opensource.org/licenses/MIT)
# =======================================================================

"""helper functions for the functions called by bunchdt"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from six.moves import zip_longest
import itertools
from eppy.constructions import thermal_properties
from eppy.geometry import surface as g_surface


def grouper(num, iterable, fillvalue=None):
    "Collect data into fixed-length chunks or blocks"
    # grouper(3, 'ABCDEFG', 'x') --> ABC DEF Gxx
    args = [iter(iterable)] * num
    return zip_longest(fillvalue=fillvalue, *args)

def getcoords(ddtt):
    """return the coordinates of the surface"""
    n_vertices_index = ddtt.objls.index('Number_of_Vertices')
    first_x = n_vertices_index + 1 # X of first coordinate
    pts = ddtt.obj[first_x:]
    return list(grouper(3, pts))

def area(ddtt):
    """area of the surface"""
    coords = getcoords(ddtt)
    return g_surface.area(coords)

def height(ddtt):
    """height of the surface"""
    coords = getcoords(ddtt)
    return g_surface.height(coords)

def width(ddtt):
    """width of the surface"""
    coords = getcoords(ddtt)
    return g_surface.width(coords)

def azimuth(ddtt):
    """azimuth of the surface"""
    coords = getcoords(ddtt)
    return g_surface.azimuth(coords)

def tilt(ddtt):
    """tilt of the surface"""
    coords = getcoords(ddtt)
    return g_surface.tilt(coords)

def buildingname(ddtt):
    """return building name"""
    idf = ddtt.theidf
    building = idf.idfobjects['building'.upper()][0]
    return building.Name
    
def zonesurfaces(ddtt):
    """return al list of surfaces that belong to the zone"""
    kwargs = {'fields':[u'Zone_Name', ],
        'iddgroups':[u'Thermal Zones and Surfaces', ]}
    return ddtt.getreferingobjs(**kwargs)
    
def subsurfaces(ddtt):
    """return al list of surfaces that belong to the zone"""
    kwargs = {'fields':[u'Building_Surface_Name', ],
        'iddgroups':[u'Thermal Zones and Surfaces', ]}
    return ddtt.getreferingobjs(**kwargs)
    
def rvalue(ddtt):
    rvalue = thermal_properties.rvalue(ddtt)
    return rvalue

def ufactor(ddtt):
    ufactor = thermal_properties.ufactor(ddtt)
    return ufactor

def ufactor_ip(ddtt):
    ufactor_ip = thermal_properties.ufactor_ip(ddtt)
    return ufactor_ip

def rvalue_ip(ddtt):
    rvalue_ip = thermal_properties.rvalue_ip(ddtt)
    return rvalue_ip

def heatcapacity(ddtt):
    heatcapacity = thermal_properties.heatcapacity(ddtt)
    return heatcapacity




# Copyright (c) 2012 Santosh Philip
# =======================================================================
#  Distributed under the MIT License.
#  (See accompanying file LICENSE or copy at
#  http://opensource.org/licenses/MIT)
# =======================================================================

"""idd comments have gaps in them.
With \note fields as indicated
This code fills those gaps
see: SCHEDULE:DAY:LIST as an example"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

# see iddgaps6.py on usage
# TODO : need unit tests for all htese functions


# works for all objects, including troublesome ones
# factor the code i  iddgaps4.py into functions


# idd might look like this
# <snip>
# / field varA 1
# / field varB 1
# / field varC 1
# / field varA 2
# / field varB 2
# / field varC 2
# / above pattern continues
#
# find objects where the fields are not named
# do the following only for those objects
# find the first field that has an integer. This is a repeating field
# gather the repeating field names (without the integer)
# generate all the repeating fields for all variables






import eppy.bunchhelpers as bunchhelpers

def cleaniddfield(acomm):
    """make all the keys lower case"""
    for key in list(acomm.keys()):
        val = acomm[key]
        acomm[key.lower()] = val
    for key in list(acomm.keys()):
        val = acomm[key]
        if key != key.lower():
            acomm.pop(key)
    return acomm

def cleancommdct(commdct):
    """make all keys in commdct lower case"""
    return [[cleaniddfield(fcomm) for fcomm in comm] for comm in commdct]

def getfields(comm):
    """get all the fields that have the key 'field' """
    fields = []
    for field in comm:
        if 'field' in field:
            fields.append(field)
    return fields

def repeatingfieldsnames(fields):
    """get the names of the repeating fields"""
    fnames = [field['field'][0] for field in fields]
    fnames = [bunchhelpers.onlylegalchar(fname) for fname in fnames]
    fnames = [fname for fname in fnames if bunchhelpers.intinlist(fname.split())]
    fnames = [(bunchhelpers.replaceint(fname), None) for fname in fnames]
    dct = dict(fnames)
    repnames = fnames[:len(list(dct.keys()))]
    return repnames

# TODO : looks like "TABLE:MULTIVARIABLELOOKUP" will have to be skipped for now.
def missingkeys_standard(commdct, dtls, skiplist=None):
    """put missing keys in commdct for standard objects
    return a list of keys where it is unable to do so
    commdct is not returned, but is updated"""
    if skiplist == None:
        skiplist = []
    # find objects where all the fields are not named
    gkeys = [dtls[i] for i in range(len(dtls)) if commdct[i].count({}) > 2]
    nofirstfields = []
    # operatie on those fields
    for key_txt in gkeys:
        if key_txt in skiplist:
            continue
        # print key_txt
        # for a function, pass comm as a variable
        key_i = dtls.index(key_txt.upper())
        comm = commdct[key_i]



        # get all fields
        fields = getfields(comm)

        # get repeating field names
        repnames = repeatingfieldsnames(fields)

        try:
            first = repnames[0][0] % (1, )
        except IndexError:
            nofirstfields.append(key_txt)
            continue
        # print first

        # get all comments of the first repeating field names
        firstnames = [repname[0] % (1, ) for repname in repnames]
        fcomments = [field for field in fields
                     if bunchhelpers.onlylegalchar(field['field'][0])
                     in firstnames]
        fcomments = [dict(fcomment) for fcomment in fcomments]
        for cmt in fcomments:
            fld = cmt['field'][0]
            fld = bunchhelpers.onlylegalchar(fld)
            fld = bunchhelpers.replaceint(fld)
            cmt['field'] = [fld]

        for i, cmt in enumerate(comm[1:]):
            thefield = cmt['field'][0]
            thefield = bunchhelpers.onlylegalchar(thefield)
            if thefield == first:
                break
        first_i = i + 1

        newfields = []
        for i in range(1, len(comm[first_i:]) // len(repnames) + 1):
            for fcomment in fcomments:
                nfcomment = dict(fcomment)
                fld = nfcomment['field'][0]
                fld = fld % (i, )
                nfcomment['field'] = [fld]
                newfields.append(nfcomment)

        for i, cmt in enumerate(comm):
            if i < first_i:
                continue
            else:
                afield = newfields.pop(0)
                comm[i] = afield
        commdct[key_i] = comm
    return nofirstfields

def missingkeys_nonstandard(block, commdct, dtls, objectlist, afield='afiled %s'):
    """This is an object list where thre is no first field name
    to give a hint of what the first field name should be"""
    afield = 'afield %s'
    for key_txt in objectlist:
        key_i = dtls.index(key_txt.upper())
        comm = commdct[key_i]
        if block:
            blk = block[key_i]
        for i, cmt in enumerate(comm):
            if cmt == {}:
                first_i = i
                break
        for i, cmt in enumerate(comm):
            if i >= first_i:
                if block:
                    comm[i]['field'] = ['%s' % (blk[i])]
                else:
                    comm[i]['field'] = [afield % (i - first_i + 1,),]



# EPlusInterface (EPI) - An interface for EnergyPlus
# Copyright (C) 2004 Santosh Philip
# =======================================================================
#  Distributed under the MIT License.
#  (See accompanying file LICENSE or copy at
#  http://opensource.org/licenses/MIT)
# =======================================================================
"""Legacy code from EPlusInterface"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

def write_str2file(pathname, astr):
    """writes a string to file"""
    fname = pathname
    fhandle = open(fname, 'wb')
    fhandle.write(astr)
    fhandle.close()

def readfile(pathname):
    """retrun the data in the file"""
    fhandle = open(pathname, 'rb')
    data = fhandle.read()
    try:
        data = data.decode('ISO-8859-2')
    except AttributeError:
        pass
    fhandle.close()
    return data

def readfileasmac():
    """docstring for readfileasmac"""
    return None


# Copyright (c) 2012 Santosh Philip
# =======================================================================
#  Distributed under the MIT License.
#  (See accompanying file LICENSE or copy at
#  http://opensource.org/licenses/MIT)
# =======================================================================
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

e = [(('p_loop Supply Inlet', 'epnode'), 'Central_Chiller'), ('Central_Chiller', ('Central_Chiller_np1_node', 'epnode')), (('Central_Chiller_np1_node', 'epnode'), 'np1'), ('np1', ('np1_np2_node', 'epnode')), (('np1_np2_node', 'epnode'), 'np2'), ('np2', ('np2_Outlet_Node_Name', 'epnode')), (('sb1_pipe_inlet', 'epnode'), 'sb1_pipe'), ('sb1_pipe', ('sb1_pipe_outlet', 'epnode')), (('sb2_pipe_inlet', 'epnode'), 'sb2_pipe'), ('sb2_pipe', ('sb2_pipe_outlet', 'epnode')), (('sb3_pipe_inlet', 'epnode'), 'sb3_pipe'), ('sb3_pipe', ('sb3_pipe_outlet', 'epnode')), (('sb4_pipe_inlet', 'epnode'), 'sb4_pipe'), ('sb4_pipe', ('p_loop Supply Outlet', 'epnode')), (('p_loop Demand Inlet', 'epnode'), 'db0_pipe'), ('db0_pipe', ('db0_pipe_outlet', 'epnode')), (('db1_pipe_inlet', 'epnode'), 'db1_pipe'), ('db1_pipe', ('db1_pipe_outlet', 'epnode')), (('db2_pipe_inlet', 'epnode'), 'db2_pipe'), ('db2_pipe', ('db2_pipe_outlet', 'epnode')), (('db3_pipe_inlet', 'epnode'), 'db3_pipe'), ('db3_pipe', ('db3_pipe_outlet', 'epnode')), (('db4_pipe_inlet', 'epnode'), 'db4_pipe'), ('db4_pipe', ('p_loop Demand Outlet', 'epnode')), (('np2_Outlet_Node_Name', 'epnode'), 'p_loop_supply_splitter'), ('p_loop_supply_splitter', ('sb1_pipe_inlet', 'epnode')), ('p_loop_supply_splitter', ('sb2_pipe_inlet', 'epnode')), ('p_loop_supply_splitter', ('sb3_pipe_inlet', 'epnode')), (('db0_pipe_outlet', 'epnode'), 'p_loop_demand_splitter'), ('p_loop_demand_splitter', ('db1_pipe_inlet', 'epnode')), ('p_loop_demand_splitter', ('db2_pipe_inlet', 'epnode')), ('p_loop_demand_splitter', ('db3_pipe_inlet', 'epnode')), ('p_loop_supply_mixer', ('sb4_pipe_inlet', 'epnode')), (('sb1_pipe_outlet', 'epnode'), 'p_loop_supply_mixer'), (('sb2_pipe_outlet', 'epnode'), 'p_loop_supply_mixer'), (('sb3_pipe_outlet', 'epnode'), 'p_loop_supply_mixer'), ('p_loop_demand_mixer', ('db4_pipe_inlet', 'epnode')), (('db1_pipe_outlet', 'epnode'), 'p_loop_demand_mixer'), (('db2_pipe_outlet', 'epnode'), 'p_loop_demand_mixer'), (('db3_pipe_outlet', 'epnode'), 'p_loop_demand_mixer')]



def nextnode(edges, component):
    """get the next component in the loop"""
    e = edges
    c = component
    n2c = [(a, b) for a, b in e if type(a) == tuple]
    c2nodes = [(a, b) for a, b in e if a == c]
    node2cs = []
    for c2node in c2nodes:
        node2c = [(a, b) for a, b in n2c if a == c2node[-1]]
        if len(node2c) == 0:
            return []
        node2cs.append(node2c[0])
    cs = [b for a, b in node2cs]
    return cs

def prevnode(edges, component):
    """get the pervious component in the loop"""
    e = edges
    c = component
    n2c = [(a, b) for a, b in e if type(a) == tuple]
    c2n = [(a, b) for a, b in e if type(b) == tuple]
    node2cs = [(a, b) for a, b in e if b == c]
    c2nodes = []
    for node2c in node2cs:
        c2node = [(a, b) for a, b in c2n if b == node2c[0]]
        if len(c2node) == 0:
            return []
        c2nodes.append(c2node[0])
    cs = [a for a, b in c2nodes]
    return cs

def main():
    edges = e
    # c = 'p_loop_supply_splitter'
    # print next(edges, component)

    c = 'Central_Chiller'

    n = 0
    while n == 0:
        print(c)
        nextcs = nextnode(e, c)
        if len(nextcs) == 0:
            break
        c = nextcs[0]
        # next(e, c)

    print('-')

    c = "sb4_pipe"
    n = 0
    while n == 0:
        print(c)
        prevcs = prevnode(e, c)
        if len(prevcs) == 0:
            break
        c = prevcs[0]
        # next(e, c)

if __name__ == '__main__':
    main()



"""current idd"""
# =======================================================================
#  Distributed under the MIT License.
#  (See accompanying file LICENSE or copy at
#  http://opensource.org/licenses/MIT)
# =======================================================================
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import eppy.iddv800 as iddcurrent
# import iddv7 as iddcurrent



# Copyright (c) 2012 Santosh Philip
# =======================================================================
#  Distributed under the MIT License.
#  (See accompanying file LICENSE or copy at
#  http://opensource.org/licenses/MIT)
# =======================================================================

"""make plant loop snippets"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import copy

import eppy.bunch_subclass as bunch_subclass
from eppy.modeleditor import IDF
import eppy.modeleditor as modeleditor
from six.moves import xrange


class WhichLoopError(Exception):
    pass

class SomeFields(object):
    """Some fields"""
    c_fields = ['Condenser Side Inlet Node Name',
                'Condenser Side Outlet Node Name',
                'Condenser Side Branch List Name',
                'Condenser Side Connector List Name',
                'Demand Side Inlet Node Name',
                'Demand Side Outlet Node Name',
                'Condenser Demand Side Branch List Name',
                'Condenser Demand Side Connector List Name']
    p_fields = ['Plant Side Inlet Node Name',
                'Plant Side Outlet Node Name',
                'Plant Side Branch List Name',
                'Plant Side Connector List Name',
                'Demand Side Inlet Node Name',
                'Demand Side Outlet Node Name',
                'Demand Side Branch List Name',
                'Demand Side Connector List Name']
    a_fields = ['Branch List Name',
                'Connector List Name',
                'Supply Side Inlet Node Name',
                'Demand Side Outlet Node Name',
                'Demand Side Inlet Node Names',
                'Supply Side Outlet Node Names']



def flattencopy(lst):
    """flatten and return a copy of the list
    indefficient on large lists"""
    # modified from
    # http://stackoverflow.com/questions/2158395/flatten-an-irregular-list-of-lists-in-python
    thelist = copy.deepcopy(lst)
    list_is_nested = True
    while list_is_nested:  # outer loop
        keepchecking = False
        atemp = []
        for element in thelist:  # inner loop
            if isinstance(element, list):
                atemp.extend(element)
                keepchecking = True
            else:
                atemp.append(element)
        list_is_nested = keepchecking  # determine if outer loop exits
        thelist = atemp[:]
    return thelist

def makepipecomponent(idf, pname):
    """make a pipe component
    generate inlet outlet names"""
    apipe = idf.newidfobject("Pipe:Adiabatic".upper(), Name=pname)
    apipe.Inlet_Node_Name = "%s_inlet" % (pname,)
    apipe.Outlet_Node_Name = "%s_outlet" % (pname,)
    return apipe

def makeductcomponent(idf, dname):
    """make a duct component
    generate inlet outlet names"""
    aduct = idf.newidfobject("duct".upper(), Name=dname)
    aduct.Inlet_Node_Name = "%s_inlet" % (dname,)
    aduct.Outlet_Node_Name = "%s_outlet" % (dname,)
    return aduct

def makepipebranch(idf, bname):
    """make a branch with a pipe
    use standard inlet outlet names"""
    # make the pipe component first
    pname = "%s_pipe" % (bname,)
    apipe = makepipecomponent(idf, pname)
    # now make the branch with the pipe in it
    abranch = idf.newidfobject("BRANCH", Name=bname)
    abranch.Component_1_Object_Type = 'Pipe:Adiabatic'
    abranch.Component_1_Name = pname
    abranch.Component_1_Inlet_Node_Name = apipe.Inlet_Node_Name
    abranch.Component_1_Outlet_Node_Name = apipe.Outlet_Node_Name
    abranch.Component_1_Branch_Control_Type = "Bypass"
    return abranch

def makeductbranch(idf, bname):
    """make a branch with a duct
    use standard inlet outlet names"""
    # make the duct component first
    pname = "%s_duct" % (bname,)
    aduct = makeductcomponent(idf, pname)
    # now make the branch with the duct in it
    abranch = idf.newidfobject("BRANCH", Name=bname)
    abranch.Component_1_Object_Type = 'duct'
    abranch.Component_1_Name = pname
    abranch.Component_1_Inlet_Node_Name = aduct.Inlet_Node_Name
    abranch.Component_1_Outlet_Node_Name = aduct.Outlet_Node_Name
    abranch.Component_1_Branch_Control_Type = "Bypass"
    return abranch

def getbranchcomponents(idf, branch, utest=False):
    """get the components of the branch"""
    fobjtype = 'Component_%s_Object_Type'
    fobjname = 'Component_%s_Name'
    complist = []
    for i in range(1, 100000):
        try:
            objtype = branch[fobjtype % (i,)]
            if objtype.strip() == '':
                break
            objname = branch[fobjname % (i,)]
            complist.append((objtype, objname))
        except bunch_subclass.BadEPFieldError:
            break
    if utest:
        return complist
    else:
        return [idf.getobject(ot, on) for ot, on in complist]

def renamenodes(idf, fieldtype):
    """rename all the changed nodes"""
    renameds = []
    for key in idf.model.dtls:
        for idfobject in idf.idfobjects[key]:
            for fieldvalue in idfobject.obj:
                if type(fieldvalue) is list:
                    if fieldvalue not in renameds:
                        cpvalue = copy.copy(fieldvalue)
                        renameds.append(cpvalue)

    # do the renaming
    for key in idf.model.dtls:
        for idfobject in idf.idfobjects[key]:
            for i, fieldvalue in enumerate(idfobject.obj):
                itsidd = idfobject.objidd[i]
                if 'type' in itsidd:
                    if itsidd['type'][0] == fieldtype:
                        tempdct = dict(renameds)
                        if type(fieldvalue) is list:
                            fieldvalue = fieldvalue[-1]
                            idfobject.obj[i] = fieldvalue
                        else:
                            if fieldvalue in tempdct:
                                fieldvalue = tempdct[fieldvalue]
                                idfobject.obj[i] = fieldvalue

def getfieldnamesendswith(idfobject, endswith):
    """get the filednames for the idfobject based on endswith"""
    objls = idfobject.objls
    tmp = [name for name in objls if name.endswith(endswith)]
    if tmp == []:
        pass
    return [name for name in objls if name.endswith(endswith)]

def getnodefieldname(idfobject, endswith, fluid=None, startswith=None):
    """return the field name of the node
    fluid is only needed if there are air and water nodes
    fluid is Air or Water or ''.
    if the fluid is Steam, use Water"""
    if startswith is None:
        startswith = ''
    if fluid is None:
        fluid = ''
    nodenames = getfieldnamesendswith(idfobject, endswith)
    nodenames = [name for name in nodenames if name.startswith(startswith)]
    fnodenames = [nd for nd in nodenames if nd.find(fluid) != -1]
    fnodenames = [name for name in fnodenames if name.startswith(startswith)]
    if len(fnodenames) == 0:
        nodename = nodenames[0]
    else:
        nodename = fnodenames[0]
    return nodename


def connectcomponents(idf, components, fluid=None):
    """rename nodes so that the components get connected
    fluid is only needed if there are air and water nodes
    fluid is Air or Water or ''.
    if the fluid is Steam, use Water"""
    if fluid is None:
        fluid = ''
    if len(components) == 1:
        thiscomp, thiscompnode = components[0]
        initinletoutlet(idf, thiscomp, thiscompnode, force=False)
        outletnodename = getnodefieldname(thiscomp, "Outlet_Node_Name",
                                          fluid=fluid, startswith=thiscompnode)
        thiscomp[outletnodename] = [thiscomp[outletnodename],
                                    thiscomp[outletnodename]]
        # inletnodename = getnodefieldname(nextcomp, "Inlet_Node_Name", fluid)
        # nextcomp[inletnodename] = [nextcomp[inletnodename], betweennodename]
        return components
    for i in range(len(components) - 1):
        thiscomp, thiscompnode = components[i]
        nextcomp, nextcompnode = components[i + 1]
        initinletoutlet(idf, thiscomp, thiscompnode, force=False)
        initinletoutlet(idf, nextcomp, nextcompnode, force=False)
        betweennodename = "%s_%s_node" % (thiscomp.Name, nextcomp.Name)
        outletnodename = getnodefieldname(thiscomp, "Outlet_Node_Name",
                                          fluid=fluid, startswith=thiscompnode)
        thiscomp[outletnodename] = [thiscomp[outletnodename], betweennodename]
        inletnodename = getnodefieldname(nextcomp, "Inlet_Node_Name", fluid)
        nextcomp[inletnodename] = [nextcomp[inletnodename], betweennodename]
    return components


def initinletoutlet(idf, idfobject, thisnode, force=False):
    """initialze values for all the inlet outlet nodes for the object.
    if force == False, it willl init only if field = '' """
    def blankfield(fieldvalue):
        """test for blank field"""
        try:
            if fieldvalue.strip() == '':
                return True
            else:
                return False
        except AttributeError:  # field may be a list
            return False
    def trimfields(fields, thisnode):
        if len(fields) > 1:
            if thisnode is not None:
                fields = [field for field in fields
                          if field.startswith(thisnode)]
                return fields
            else:
                print("Where should this loop connect ?")
                print("%s - %s" % (idfobject.key, idfobject.Name))
                print([field.split("Inlet_Node_Name")[0]
                       for field in inletfields])
                raise WhichLoopError
        else:
            return fields

    inletfields = getfieldnamesendswith(idfobject, "Inlet_Node_Name")
    inletfields = trimfields(inletfields, thisnode)  # or warn with exception
    for inletfield in inletfields:
        if blankfield(idfobject[inletfield]) == True or force == True:
            idfobject[inletfield] = "%s_%s" % (idfobject.Name, inletfield)
    outletfields = getfieldnamesendswith(idfobject, "Outlet_Node_Name")
    outletfields = trimfields(outletfields, thisnode)  # or warn with exception
    for outletfield in outletfields:
        if blankfield(idfobject[outletfield]) == True or force == True:
            idfobject[outletfield] = "%s_%s" % (idfobject.Name, outletfield)
    return idfobject

def componentsintobranch(idf, branch, listofcomponents, fluid=None):
    """insert a list of components into a branch
    fluid is only needed if there are air and water nodes in same object
    fluid is Air or Water or ''.
    if the fluid is Steam, use Water"""
    if fluid is None:
        fluid = ''
    componentlist = [item[0] for item in listofcomponents]
    # assumes that the nodes of the component connect to each other
    # empty branch if it has existing components
    thebranchname = branch.Name
    thebranch = idf.removeextensibles('BRANCH', thebranchname)  # empty the branch
    # fill in the new components with the node names into this branch
        # find the first extensible field and fill in the data in obj.
    e_index = idf.getextensibleindex('BRANCH', thebranchname)
    theobj = thebranch.obj
    modeleditor.extendlist(theobj, e_index)  # just being careful here
    for comp, compnode in listofcomponents:
        theobj.append(comp.key)
        theobj.append(comp.Name)
        inletnodename = getnodefieldname(comp, "Inlet_Node_Name", fluid=fluid,
                                         startswith=compnode)
        theobj.append(comp[inletnodename])
        outletnodename = getnodefieldname(comp, "Outlet_Node_Name",
                                          fluid=fluid, startswith=compnode)
        theobj.append(comp[outletnodename])
        theobj.append('')

    return thebranch

def doingtesting(testing, testn, result=None):
    """doing testing"""
    testn += 1
    if testing == testn:
        print(testing)
        returnnone()
    else:
        return testn

def returnnone():
    """return None"""
    return None

def makeairloop(idf, loopname, sloop, dloop, testing=None):
    """make an airloop"""
    # -------- testing ---------
    testn = 0
    # -------- testing ---------
    newairloop = idf.newidfobject("AirLoopHVAC".upper(), Name=loopname)
    # -------- testing ---------
    testn = doingtesting(testing, testn, newairloop)
    if testn == None:
        returnnone()
    # -------- testing ---------
    fields = SomeFields.a_fields

    # for use in bunch
    flnames = [field.replace(' ', '_') for field in fields]

    # simplify naming
    fields1 = ['Branches',
               'Connectors',
               'Supply Inlet',
               'Demand Outlet',
               'Demand Inlet',
               'Supply Outlet']

    # old TODO : pop connectors if no parallel branches
    # make fieldnames in the air loop
    fieldnames = ['%s %s' % (loopname, field) for field in fields1]
    for fieldname, thefield in zip(fieldnames, flnames):
        newairloop[thefield] = fieldname
    # -------- testing ---------
    testn = doingtesting(testing, testn, newairloop)
    if testn == None:
        returnnone()
    # -------- testing ---------

    # make the branch lists for this air loop
    sbranchlist = idf.newidfobject("BRANCHLIST",
                                   Name=newairloop[flnames[0]])

    # -------- testing ---------
    testn = doingtesting(testing, testn, newairloop)
    if testn == None:
        returnnone()
    # -------- testing ---------
    # add branch names to the branchlist
    sbranchnames = flattencopy(sloop)
    # sbranchnames = sloop[1]
    for branchname in sbranchnames:
        sbranchlist.obj.append(branchname)
    # -------- testing ---------
    testn = doingtesting(testing, testn, newairloop)
    if testn == None:
        returnnone()
    # -------- testing ---------
    # supply side
    sbranchs = []
    for bname in sbranchnames:
        branch = makeductbranch(idf, bname)
        sbranchs.append(branch)
    # -------- testing ---------
    testn = doingtesting(testing, testn, newairloop)
    if testn == None:
        returnnone()
    # -------- testing ---------
    # rename inlet outlet of endpoints of loop
    anode = "Component_1_Inlet_Node_Name"
    sameinnode = "Supply_Side_Inlet_Node_Name"  # TODO : change ?
    sbranchs[0][anode] = newairloop[sameinnode]
    anode = "Component_1_Outlet_Node_Name"
    sameoutnode = "Supply_Side_Outlet_Node_Names"  # TODO : change ?
    sbranchs[-1][anode] = newairloop[sameoutnode]
    # -------- testing ---------
    testn = doingtesting(testing, testn, newairloop)
    if testn == None:
        returnnone()
    # -------- testing ---------
    # rename inlet outlet of endpoints of loop - rename in pipe
    dname = sbranchs[0]['Component_1_Name']  # get the duct name
    aduct = idf.getobject('duct'.upper(), dname)  # get duct
    aduct.Inlet_Node_Name = newairloop[sameinnode]
    dname = sbranchs[-1]['Component_1_Name']  # get the duct name
    aduct = idf.getobject('duct'.upper(), dname)  # get duct
    aduct.Outlet_Node_Name = newairloop[sameoutnode]
    # -------- testing ---------
    testn = doingtesting(testing, testn, newairloop)
    if testn == None:
        returnnone()
    # -------- testing ---------
    #
    # # TODO : test if there are parallel branches
    # make the connectorlist an fill fields
    sconnlist = idf.newidfobject("CONNECTORLIST",
                                 Name=newairloop.Connector_List_Name)
    # -------- testing ---------
    testn = doingtesting(testing, testn, newairloop)
    if testn == None:
        returnnone()
    # -------- testing ---------
    sconnlist.Connector_1_Object_Type = "Connector:Splitter"
    sconnlist.Connector_1_Name = "%s_supply_splitter" % (loopname,)
    sconnlist.Connector_2_Object_Type = "Connector:Mixer"
    sconnlist.Connector_2_Name = "%s_supply_mixer" % (loopname,)
    # -------- testing ---------
    testn = doingtesting(testing, testn, newairloop)
    if testn == None:
        returnnone()
    # -------- testing ---------
    # make splitters and mixers
    s_splitter = idf.newidfobject("CONNECTOR:SPLITTER",
                                  Name=sconnlist.Connector_1_Name)
    s_splitter.obj.extend([sloop[0]] + sloop[1])
    s_mixer = idf.newidfobject("CONNECTOR:MIXER",
                               Name=sconnlist.Connector_2_Name)
    s_mixer.obj.extend([sloop[-1]] + sloop[1])
    # -------- testing ---------
    testn = doingtesting(testing, testn, newairloop)
    if testn == None:
        returnnone()
    # -------- testing ---------
    # demand side loop for airloop is made below
    # ZoneHVAC:EquipmentConnections
    for zone in dloop:
        equipconn = idf.newidfobject("ZoneHVAC:EquipmentConnections".upper())
        equipconn.Zone_Name = zone
        fldname = "Zone_Conditioning_Equipment_List_Name"
        equipconn[fldname] = "%s equip list" % (zone,)
        fldname = "Zone_Air_Inlet_Node_or_NodeList_Name"
        equipconn[fldname] = "%s Inlet Node" % (zone,)
        fldname = "Zone_Air_Node_Name"
        equipconn[fldname] = "%s Node" % (zone,)
        fldname = "Zone_Return_Air_Node_Name"
        equipconn[fldname] = "%s Outlet Node" % (zone,)
    # -------- testing ---------
    testn = doingtesting(testing, testn, newairloop)
    if testn == None:
        returnnone()
    # -------- testing ---------
    # make ZoneHVAC:EquipmentList
    for zone in dloop:
        z_equiplst = idf.newidfobject("ZoneHVAC:EquipmentList".upper())
        z_equipconn = modeleditor.getobjects(
            idf.idfobjects,
            idf.model, idf.idd_info,
            "ZoneHVAC:EquipmentConnections".upper(),  # places=7,
            **dict(Zone_Name=zone))[0]
        z_equiplst.Name = z_equipconn.Zone_Conditioning_Equipment_List_Name
        fld = "Zone_Equipment_1_Object_Type"
        z_equiplst[fld] = "AirTerminal:SingleDuct:Uncontrolled"
        z_equiplst.Zone_Equipment_1_Name = "%sDirectAir" % (zone,)
        z_equiplst.Zone_Equipment_1_Cooling_Sequence = 1
        z_equiplst.Zone_Equipment_1_Heating_or_NoLoad_Sequence = 1
    # -------- testing ---------
    testn = doingtesting(testing, testn, newairloop)
    if testn == None:
        returnnone()
    # -------- testing ---------
    # make AirTerminal:SingleDuct:Uncontrolled
    for zone in dloop:
        z_equipconn = modeleditor.getobjects(
            idf.idfobjects,
            idf.model, idf.idd_info,
            "ZoneHVAC:EquipmentConnections".upper(),  # places=7,
            **dict(Zone_Name=zone))[0]
        key = "AirTerminal:SingleDuct:Uncontrolled".upper()
        z_airterm = idf.newidfobject(key)
        z_airterm.Name = "%sDirectAir" % (zone,)
        fld1 = "Zone_Supply_Air_Node_Name"
        fld2 = "Zone_Air_Inlet_Node_or_NodeList_Name"
        z_airterm[fld1] = z_equipconn[fld2]
        z_airterm.Maximum_Air_Flow_Rate = 'autosize'
    # -------- testing ---------
    testn = doingtesting(testing, testn, newairloop)
    if testn == None:
        returnnone()
    # -------- testing ---------
    # MAKE AirLoopHVAC:ZoneSplitter
    # zone = dloop[0]
    key = "AirLoopHVAC:ZoneSplitter".upper()
    z_splitter = idf.newidfobject(key)
    # -------- testing ---------
    testn = doingtesting(testing, testn, newairloop)
    if testn == None:
        returnnone()
    # -------- testing ---------
    z_splitter.Name = "%s Demand Side Splitter" % (loopname,)
    z_splitter.Inlet_Node_Name = newairloop.Demand_Side_Inlet_Node_Names
    for i, zone in enumerate(dloop):
        z_equipconn = modeleditor.getobjects(
            idf.idfobjects,
            idf.model, idf.idd_info,
            "ZoneHVAC:EquipmentConnections".upper(),  # places=7,
            **dict(Zone_Name=zone))[0]
        fld = "Outlet_%s_Node_Name" % (i + 1,)
        z_splitter[fld] = z_equipconn.Zone_Air_Inlet_Node_or_NodeList_Name
    # -------- testing ---------
    testn = doingtesting(testing, testn, newairloop)
    if testn == None:
        returnnone()
    # -------- testing ---------
    # make AirLoopHVAC:SupplyPath
    key = "AirLoopHVAC:SupplyPath".upper()
    z_supplypth = idf.newidfobject(key)
    z_supplypth.Name = "%sSupplyPath" % (loopname,)
    # -------- testing ---------
    testn = doingtesting(testing, testn, newairloop)
    if testn == None:
        returnnone()
    # -------- testing ---------
    fld1 = "Supply_Air_Path_Inlet_Node_Name"
    fld2 = "Demand_Side_Inlet_Node_Names"
    z_supplypth[fld1] = newairloop[fld2]
    z_supplypth.Component_1_Object_Type = "AirLoopHVAC:ZoneSplitter"
    z_supplypth.Component_1_Name = z_splitter.Name
    # -------- testing ---------
    testn = doingtesting(testing, testn, newairloop)
    if testn == None:
        returnnone()
    # -------- testing ---------
    # make AirLoopHVAC:ZoneMixer
    key = "AirLoopHVAC:ZoneMixer".upper()
    z_mixer = idf.newidfobject(key)
    # -------- testing ---------
    testn = doingtesting(testing, testn, newairloop)
    if testn == None:
        returnnone()
    # -------- testing ---------
    z_mixer.Name = "%s Demand Side Mixer" % (loopname,)
    # -------- testing ---------
    testn = doingtesting(testing, testn, newairloop)
    if testn == None:
        returnnone()
    # -------- testing ---------
    z_mixer.Outlet_Node_Name = newairloop.Demand_Side_Outlet_Node_Name
    # -------- testing ---------
    testn = doingtesting(testing, testn, newairloop)
    if testn == None:
        returnnone()
    # -------- testing ---------
    for i, zone in enumerate(dloop):
        z_equipconn = modeleditor.getobjects(
            idf.idfobjects,
            idf.model, idf.idd_info,
            "ZoneHVAC:EquipmentConnections".upper(),  # places=7,
            **dict(Zone_Name=zone))[0]
        fld = "Inlet_%s_Node_Name" % (i + 1,)
        z_mixer[fld] = z_equipconn.Zone_Return_Air_Node_Name
    # -------- testing ---------
    testn = doingtesting(testing, testn, newairloop)
    if testn == None:
        returnnone()
    # -------- testing ---------
    # make AirLoopHVAC:ReturnPath
    key = "AirLoopHVAC:ReturnPath".upper()
    z_returnpth = idf.newidfobject(key)
    # -------- testing ---------
    testn = doingtesting(testing, testn, newairloop)
    if testn == None:
        returnnone()
    # -------- testing ---------
    z_returnpth.Name = "%sReturnPath" % (loopname,)
    z_returnpth.Return_Air_Path_Outlet_Node_Name = newairloop.Demand_Side_Outlet_Node_Name
    z_returnpth.Component_1_Object_Type = "AirLoopHVAC:ZoneMixer"
    z_returnpth.Component_1_Name = z_mixer.Name
    # -------- testing ---------
    testn = doingtesting(testing, testn, newairloop)
    if testn == None:
        returnnone()
    # -------- testing ---------
    return newairloop

def makeplantloop(idf, loopname, sloop, dloop, testing=None):
    """make plant loop with pip components"""
    # -------- <testing ---------
    testn = 0
    # -------- testing> ---------
    newplantloop = idf.newidfobject("PLANTLOOP", Name=loopname)
    # -------- <testing ---------
    testn = doingtesting(testing, testn, newplantloop)
    if testn == None:
        returnnone()
    # -------- testing> ---------
    fields = SomeFields.p_fields

    # for use in bunch
    flnames = [field.replace(' ', '_') for field in fields]

    # simplify naming
    fields1 = [field.replace('Plant Side', 'Supply') for field in fields]
    fields1 = [field.replace('Demand Side', 'Demand') for field in fields1]
    fields1 = [field[:field.find('Name') - 1] for field in fields1]
    fields1 = [field.replace(' Node', '') for field in fields1]
    fields1 = [field.replace(' List', 's') for field in fields1]

    # TODO : pop connectors if no parallel branches
    # make fieldnames in the plant loop
    fieldnames = ['%s %s' % (loopname, field) for field in fields1]
    for fieldname, thefield in zip(fieldnames, flnames):
        newplantloop[thefield] = fieldname
    # -------- <testing ---------
    testn = doingtesting(testing, testn, newplantloop)
    if testn == None:
        returnnone()
    # -------- testing> ---------

    # make the branch lists for this plant loop
    sbranchlist = idf.newidfobject(
        "BRANCHLIST",
        Name=newplantloop.Plant_Side_Branch_List_Name)
    # -------- <testing ---------
    testn = doingtesting(testing, testn, newplantloop)
    if testn == None:
        returnnone()
    # -------- testing> ---------
    dbranchlist = idf.newidfobject(
        "BRANCHLIST",
        Name=newplantloop.Demand_Side_Branch_List_Name)
    # -------- <testing ---------
    testn = doingtesting(testing, testn, newplantloop)
    if testn == None:
        returnnone()
    # -------- testing> ---------
    # add branch names to the branchlist
    sbranchnames = flattencopy(sloop)
    # sbranchnames = sloop[1]
    for branchname in sbranchnames:
        sbranchlist.obj.append(branchname)
    # -------- <testing ---------
    testn = doingtesting(testing, testn, newplantloop)
    if testn == None:
        returnnone()
    # -------- testing> ---------
    dbranchnames = flattencopy(dloop)
    # dbranchnames = dloop[1]
    for branchname in dbranchnames:
        dbranchlist.obj.append(branchname)
    # -------- <testing ---------
    testn = doingtesting(testing, testn, newplantloop)
    if testn == None:
        returnnone()
    # -------- testing> ---------

    # make a pipe branch for all branches in the loop

    # supply side
    sbranchs = []
    for bname in sbranchnames:
        branch = makepipebranch(idf, bname)
        sbranchs.append(branch)
    # -------- <testing ---------
    testn = doingtesting(testing, testn, newplantloop)
    if testn == None:
        returnnone()
    # -------- testing> ---------
    # rename inlet outlet of endpoints of loop
    anode = "Component_1_Inlet_Node_Name"
    sameinnode = "Plant_Side_Inlet_Node_Name"
    sbranchs[0][anode] = newplantloop[sameinnode]
    anode = "Component_1_Outlet_Node_Name"
    sameoutnode = "Plant_Side_Outlet_Node_Name"
    sbranchs[-1][anode] = newplantloop[sameoutnode]
    # -------- <testing ---------
    testn = doingtesting(testing, testn, newplantloop)
    if testn == None:
        returnnone()
    # -------- testing> ---------
    # rename inlet outlet of endpoints of loop - rename in pipe
    pname = sbranchs[0]['Component_1_Name']  # get the pipe name
    apipe = idf.getobject('Pipe:Adiabatic'.upper(), pname)  # get pipe
    apipe.Inlet_Node_Name = newplantloop[sameinnode]
    pname = sbranchs[-1]['Component_1_Name']  # get the pipe name
    apipe = idf.getobject('Pipe:Adiabatic'.upper(), pname)  # get pipe
    apipe.Outlet_Node_Name = newplantloop[sameoutnode]
    # -------- <testing ---------
    testn = doingtesting(testing, testn, newplantloop)
    if testn == None:
        returnnone()
    # -------- testing> ---------

    # demand side
    dbranchs = []
    for bname in dbranchnames:
        branch = makepipebranch(idf, bname)
        dbranchs.append(branch)
    # -------- <testing ---------
    testn = doingtesting(testing, testn, newplantloop)
    if testn == None:
        returnnone()
    # -------- testing> ---------
    # rename inlet outlet of endpoints of loop - rename in branch
    anode = "Component_1_Inlet_Node_Name"
    sameinnode = "Demand_Side_Inlet_Node_Name"
    dbranchs[0][anode] = newplantloop[sameinnode]
    anode = "Component_1_Outlet_Node_Name"
    sameoutnode = "Demand_Side_Outlet_Node_Name"
    dbranchs[-1][anode] = newplantloop[sameoutnode]
    # -------- <testing ---------
    testn = doingtesting(testing, testn, newplantloop)
    if testn == None:
        returnnone()
    # -------- testing> ---------
    # rename inlet outlet of endpoints of loop - rename in pipe
    pname = dbranchs[0]['Component_1_Name']  # get the pipe name
    apipe = idf.getobject('Pipe:Adiabatic'.upper(), pname)  # get pipe
    apipe.Inlet_Node_Name = newplantloop[sameinnode]
    pname = dbranchs[-1]['Component_1_Name']  # get the pipe name
    apipe = idf.getobject('Pipe:Adiabatic'.upper(), pname)  # get pipe
    apipe.Outlet_Node_Name = newplantloop[sameoutnode]
    # -------- <testing ---------
    testn = doingtesting(testing, testn, newplantloop)
    if testn == None:
        returnnone()
    # -------- testing> ---------


    # TODO : test if there are parallel branches
    # make the connectorlist an fill fields
    sconnlist = idf.newidfobject(
        "CONNECTORLIST",
        Name=newplantloop.Plant_Side_Connector_List_Name)
    sconnlist.Connector_1_Object_Type = "Connector:Splitter"
    sconnlist.Connector_1_Name = "%s_supply_splitter" % (loopname,)
    sconnlist.Connector_2_Object_Type = "Connector:Mixer"
    sconnlist.Connector_2_Name = "%s_supply_mixer" % (loopname,)
    dconnlist = idf.newidfobject(
        "CONNECTORLIST",
        Name=newplantloop.Demand_Side_Connector_List_Name)
    dconnlist.Connector_1_Object_Type = "Connector:Splitter"
    dconnlist.Connector_1_Name = "%s_demand_splitter" % (loopname,)
    dconnlist.Connector_2_Object_Type = "Connector:Mixer"
    dconnlist.Connector_2_Name = "%s_demand_mixer" % (loopname,)
    # -------- <testing ---------
    testn = doingtesting(testing, testn, newplantloop)
    if testn == None:
        returnnone()
    # -------- testing> ---------

    # make splitters and mixers
    s_splitter = idf.newidfobject(
        "CONNECTOR:SPLITTER",
        Name=sconnlist.Connector_1_Name)
    s_splitter.obj.extend([sloop[0]] + sloop[1])
    s_mixer = idf.newidfobject(
        "CONNECTOR:MIXER",
        Name=sconnlist.Connector_2_Name)
    s_mixer.obj.extend([sloop[-1]] + sloop[1])
    # -
    d_splitter = idf.newidfobject(
        "CONNECTOR:SPLITTER",
        Name=dconnlist.Connector_1_Name)
    d_splitter.obj.extend([dloop[0]] + dloop[1])
    d_mixer = idf.newidfobject(
        "CONNECTOR:MIXER",
        Name=dconnlist.Connector_2_Name)
    d_mixer.obj.extend([dloop[-1]] + dloop[1])
    # -------- <testing ---------
    testn = doingtesting(testing, testn, newplantloop)
    if testn == None:
        returnnone()
    # -------- testing> ---------

    return newplantloop

def makecondenserloop(idf, loopname, sloop, dloop, testing=None):
    """make condenser loop with pipe components"""
    # -------- <testing ---------
    testn = 0
    # -------- testing> ---------
    newcondenserloop = idf.newidfobject("CondenserLoop".upper(), Name=loopname)
    # -------- <testing ---------
    testn = doingtesting(testing, testn, newcondenserloop)
    if testn == None:
        returnnone()
    # -------- testing> ---------

    fields = SomeFields.c_fields

    # for use in bunch
    flnames = [field.replace(' ', '_') for field in fields]

    # simplify naming
    fields1 = [field.replace(
        'Condenser Side',
        'Cond_Supply') for field in fields]
    fields1 = [field.replace('Demand Side', 'Demand') for field in fields1]
    fields1 = [field[:field.find('Name') - 1] for field in fields1]
    fields1 = [field.replace(' Node', '') for field in fields1]
    fields1 = [field.replace(' List', 's') for field in fields1]

    # old TODO : pop connectors if no parallel branches
    # make fieldnames in the condenser loop
    fieldnames = ['%s %s' % (loopname, field) for field in fields1]
    for fieldname, thefield in zip(fieldnames, flnames):
        newcondenserloop[thefield] = fieldname
    # -------- <testing ---------
    testn = doingtesting(testing, testn, newcondenserloop)
    if testn == None:
        returnnone()
    # -------- testing> ---------

    # make the branch lists for this condenser loop
    sbranchlist = idf.newidfobject(
        "BRANCHLIST",
        Name=newcondenserloop.Condenser_Side_Branch_List_Name)
    dbranchlist = idf.newidfobject(
        "BRANCHLIST",
        Name=newcondenserloop.Condenser_Demand_Side_Branch_List_Name)
    # -------- <testing ---------
    testn = doingtesting(testing, testn, newcondenserloop)
    if testn == None:
        returnnone()
    # -------- testing> ---------

    # add branch names to the branchlist
    sbranchnames = flattencopy(sloop)
    # sbranchnames = sloop[1]
    for branchname in sbranchnames:
        sbranchlist.obj.append(branchname)
    dbranchnames = flattencopy(dloop)
    # dbranchnames = dloop[1]
    for branchname in dbranchnames:
        dbranchlist.obj.append(branchname)
    # -------- <testing ---------
    testn = doingtesting(testing, testn, newcondenserloop)
    if testn == None:
        returnnone()
    # -------- testing> ---------

    # make a pipe branch for all branches in the loop

    # supply side
    sbranchs = []
    for bname in sbranchnames:
        branch = makepipebranch(idf, bname)
        sbranchs.append(branch)
    # -------- <testing ---------
    testn = doingtesting(testing, testn, newcondenserloop)
    if testn == None:
        returnnone()
    # -------- testing> ---------
    # rename inlet outlet of endpoints of loop
    anode = "Component_1_Inlet_Node_Name"
    sameinnode = "Condenser_Side_Inlet_Node_Name"  # TODO : change ?
    sbranchs[0][anode] = newcondenserloop[sameinnode]
    anode = "Component_1_Outlet_Node_Name"
    sameoutnode = "Condenser_Side_Outlet_Node_Name"  # TODO : change ?
    sbranchs[-1][anode] = newcondenserloop[sameoutnode]
    # -------- <testing ---------
    testn = doingtesting(testing, testn, newcondenserloop)
    if testn == None:
        returnnone()
    # -------- testing> ---------
    # rename inlet outlet of endpoints of loop - rename in pipe
    pname = sbranchs[0]['Component_1_Name']  # get the pipe name
    apipe = idf.getobject('Pipe:Adiabatic'.upper(), pname)  # get pipe
    apipe.Inlet_Node_Name = newcondenserloop[sameinnode]
    pname = sbranchs[-1]['Component_1_Name']  # get the pipe name
    apipe = idf.getobject('Pipe:Adiabatic'.upper(), pname)  # get pipe
    apipe.Outlet_Node_Name = newcondenserloop[sameoutnode]
    # -------- <testing ---------
    testn = doingtesting(testing, testn, newcondenserloop)
    if testn == None:
        returnnone()
    # -------- testing> ---------

    # demand side
    dbranchs = []
    for bname in dbranchnames:
        branch = makepipebranch(idf, bname)
        dbranchs.append(branch)
    # -------- <testing ---------
    testn = doingtesting(testing, testn, newcondenserloop)
    if testn == None:
        returnnone()
    # -------- testing> ---------
    # rename inlet outlet of endpoints of loop - rename in branch
    anode = "Component_1_Inlet_Node_Name"
    sameinnode = "Demand_Side_Inlet_Node_Name"  # TODO : change ?
    dbranchs[0][anode] = newcondenserloop[sameinnode]
    anode = "Component_1_Outlet_Node_Name"
    sameoutnode = "Demand_Side_Outlet_Node_Name"  # TODO : change ?
    dbranchs[-1][anode] = newcondenserloop[sameoutnode]
    # -------- <testing ---------
    testn = doingtesting(testing, testn, newcondenserloop)
    if testn == None:
        returnnone()
    # -------- testing> ---------
    # rename inlet outlet of endpoints of loop - rename in pipe
    pname = dbranchs[0]['Component_1_Name']  # get the pipe name
    apipe = idf.getobject('Pipe:Adiabatic'.upper(), pname)  # get pipe
    apipe.Inlet_Node_Name = newcondenserloop[sameinnode]
    pname = dbranchs[-1]['Component_1_Name']  # get the pipe name
    apipe = idf.getobject('Pipe:Adiabatic'.upper(), pname)  # get pipe
    apipe.Outlet_Node_Name = newcondenserloop[sameoutnode]
    # -------- <testing ---------
    testn = doingtesting(testing, testn, newcondenserloop)
    if testn == None:
        returnnone()
    # -------- testing> ---------


    # TODO : test if there are parallel branches
    # make the connectorlist an fill fields
    sconnlist = idf.newidfobject(
        "CONNECTORLIST",
        Name=newcondenserloop.Condenser_Side_Connector_List_Name)
    sconnlist.Connector_1_Object_Type = "Connector:Splitter"
    sconnlist.Connector_1_Name = "%s_supply_splitter" % (loopname,)
    sconnlist.Connector_2_Object_Type = "Connector:Mixer"
    sconnlist.Connector_2_Name = "%s_supply_mixer" % (loopname,)
    # -------- <testing ---------
    testn = doingtesting(testing, testn, newcondenserloop)
    if testn == None:
        returnnone()
    # -------- testing> ---------
    dconnlist = idf.newidfobject(
        "CONNECTORLIST",
        Name=newcondenserloop.Condenser_Demand_Side_Connector_List_Name)
    dconnlist.Connector_1_Object_Type = "Connector:Splitter"
    dconnlist.Connector_1_Name = "%s_demand_splitter" % (loopname,)
    dconnlist.Connector_2_Object_Type = "Connector:Mixer"
    dconnlist.Connector_2_Name = "%s_demand_mixer" % (loopname,)
    # -------- <testing ---------
    testn = doingtesting(testing, testn, newcondenserloop)
    if testn == None:
        returnnone()
    # -------- testing> ---------

    # make splitters and mixers
    s_splitter = idf.newidfobject(
        "CONNECTOR:SPLITTER",
        Name=sconnlist.Connector_1_Name)
    s_splitter.obj.extend([sloop[0]] + sloop[1])
    s_mixer = idf.newidfobject(
        "CONNECTOR:MIXER",
        Name=sconnlist.Connector_2_Name)
    s_mixer.obj.extend([sloop[-1]] + sloop[1])
    # -------- <testing ---------
    testn = doingtesting(testing, testn, newcondenserloop)
    if testn == None:
        returnnone()
    # -------- testing> ---------
    # -
    d_splitter = idf.newidfobject(
        "CONNECTOR:SPLITTER",
        Name=dconnlist.Connector_1_Name)
    d_splitter.obj.extend([dloop[0]] + dloop[1])
    d_mixer = idf.newidfobject(
        "CONNECTOR:MIXER",
        Name=dconnlist.Connector_2_Name)
    d_mixer.obj.extend([dloop[-1]] + dloop[1])
    # -------- <testing ---------
    testn = doingtesting(testing, testn, newcondenserloop)
    if testn == None:
        returnnone()
    # -------- testing> ---------
    return newcondenserloop

def _clean_listofcomponents(listofcomponents):
    """force it to be a list of tuples"""
    def totuple(item):
        """return a tuple"""
        if isinstance(item, (tuple, list)):
            return item
        else:
            return (item, None)
    return [totuple(item) for item in listofcomponents]

def _clean_listofcomponents_tuples(listofcomponents_tuples):
    """force 3 items in the tuple"""
    def to3tuple(item):
        """return a 3 item tuple"""
        if len(item) == 3:
            return item
        else:
            return (item[0], item[1], None)
    return [to3tuple(item) for item in listofcomponents_tuples]

def getmakeidfobject(idf, key, name):
    """get idfobject or make it if it does not exist"""
    idfobject = idf.getobject(key, name)
    if not idfobject:
        return idf.newidfobject(key, Name=name)
    else:
        return idfobject

def replacebranch1(idf, loop, branchname, listofcomponents_tuples, fluid=None,
                   debugsave=False):
    """do I even use this ? .... yup! I do"""
    if fluid is None:
        fluid = ''
    listofcomponents_tuples = _clean_listofcomponents_tuples(listofcomponents_tuples)
    branch = idf.getobject('BRANCH', branchname)  # args are (key, name)
    listofcomponents = []
    for comp_type, comp_name, compnode in listofcomponents_tuples:
        comp = getmakeidfobject(idf, comp_type.upper(), comp_name)
        listofcomponents.append((comp, compnode))
    newbr = replacebranch(idf, loop, branch, listofcomponents,
                          debugsave=debugsave, fluid=fluid)
    return newbr

def replacebranch(idf, loop, branch,
                  listofcomponents, fluid=None,
                  debugsave=False,
                  testing=None):
    """It will replace the components in the branch with components in
    listofcomponents"""
    if fluid is None:
        fluid = ''
    # -------- testing ---------
    testn = 0
    # -------- testing ---------

    # join them into a branch
    # -----------------------
    # np1_inlet -> np1 -> np1_np2_node -> np2 -> np2_outlet
        # change the node names in the component
        # empty the old branch
        # fill in the new components with the node names into this branch
    listofcomponents = _clean_listofcomponents(listofcomponents)

    components = [item[0] for item in listofcomponents]
    connectcomponents(idf, listofcomponents, fluid=fluid)
    if debugsave:
        idf.savecopy("hhh3.idf")
    # -------- testing ---------
    testn = doingtesting(testing, testn)
    if testn == None:
        returnnone()
    # -------- testing ---------
    fields = SomeFields.a_fields

    thebranch = branch
    componentsintobranch(idf, thebranch, listofcomponents, fluid=fluid)
    if debugsave:
        idf.savecopy("hhh4.idf")
    # -------- testing ---------
    testn = doingtesting(testing, testn)
    if testn == None:
        returnnone()
    # -------- testing ---------

    # # gather all renamed nodes
    # # do the renaming
    renamenodes(idf, 'node')
    if debugsave:
        idf.savecopy("hhh7.idf")
    # -------- testing ---------
    testn = doingtesting(testing, testn)
    if testn == None:
        returnnone()
    # -------- testing ---------

    # check for the end nodes of the loop
    if loop.key == 'AIRLOOPHVAC':
        fields = SomeFields.a_fields
    if loop.key == 'PLANTLOOP':
        fields = SomeFields.p_fields
    if loop.key == 'CONDENSERLOOP':
        fields = SomeFields.c_fields
    # for use in bunch
    flnames = [field.replace(' ', '_') for field in fields]

    if fluid.upper() == 'WATER':
        supplyconlistname = loop[flnames[3]]
        # Plant_Side_Connector_List_Name or Condenser_Side_Connector_List_Name
    elif fluid.upper() == 'AIR':
        supplyconlistname = loop[flnames[1]]  # Connector_List_Name'
    supplyconlist = idf.getobject('CONNECTORLIST', supplyconlistname)
    for i in range(1, 100000):  # large range to hit end
        try:
            fieldname = 'Connector_%s_Object_Type' % (i,)
            ctype = supplyconlist[fieldname]
        except bunch_subclass.BadEPFieldError:
            break
        if ctype.strip() == '':
            break
        fieldname = 'Connector_%s_Name' % (i,)
        cname = supplyconlist[fieldname]
        connector = idf.getobject(ctype.upper(), cname)
        if connector.key == 'CONNECTOR:SPLITTER':
            firstbranchname = connector.Inlet_Branch_Name
            cbranchname = firstbranchname
            isfirst = True
        if connector.key == 'CONNECTOR:MIXER':
            lastbranchname = connector.Outlet_Branch_Name
            cbranchname = lastbranchname
            isfirst = False
        if cbranchname == thebranch.Name:
            # rename end nodes
            comps = getbranchcomponents(idf, thebranch)
            if isfirst:
                comp = comps[0]
                inletnodename = getnodefieldname(
                    comp,
                    "Inlet_Node_Name", fluid)
                comp[inletnodename] = [
                    comp[inletnodename],
                    loop[flnames[0]]]  # Plant_Side_Inlet_Node_Name
            else:
                comp = comps[-1]
                outletnodename = getnodefieldname(
                    comp,
                    "Outlet_Node_Name", fluid)
                comp[outletnodename] = [
                    comp[outletnodename],
                    loop[flnames[1]]]  # .Plant_Side_Outlet_Node_Name
    # -------- testing ---------
    testn = doingtesting(testing, testn)
    if testn == None:
        returnnone()
    # -------- testing ---------

    if fluid.upper() == 'WATER':
        demandconlistname = loop[flnames[7]]  # .Demand_Side_Connector_List_Name
        demandconlist = idf.getobject('CONNECTORLIST', demandconlistname)
        for i in range(1, 100000):  # large range to hit end
            try:
                fieldname = 'Connector_%s_Object_Type' % (i,)
                ctype = demandconlist[fieldname]
            except bunch_subclass.BadEPFieldError:
                break
            if ctype.strip() == '':
                break
            fieldname = 'Connector_%s_Name' % (i,)
            cname = demandconlist[fieldname]
            connector = idf.getobject(ctype.upper(), cname)
            if connector.key == 'CONNECTOR:SPLITTER':
                firstbranchname = connector.Inlet_Branch_Name
                cbranchname = firstbranchname
                isfirst = True
            if connector.key == 'CONNECTOR:MIXER':
                lastbranchname = connector.Outlet_Branch_Name
                cbranchname = lastbranchname
                isfirst = False
            if cbranchname == thebranch.Name:
                # rename end nodes
                comps = getbranchcomponents(idf, thebranch)
                if isfirst:
                    comp = comps[0]
                    inletnodename = getnodefieldname(
                        comp,
                        "Inlet_Node_Name", fluid)
                    comp[inletnodename] = [
                        comp[inletnodename],
                        loop[flnames[4]]]  # .Demand_Side_Inlet_Node_Name
                if not isfirst:
                    comp = comps[-1]
                    outletnodename = getnodefieldname(
                        comp,
                        "Outlet_Node_Name", fluid)
                    comp[outletnodename] = [
                        comp[outletnodename],
                        loop[flnames[5]]]  # .Demand_Side_Outlet_Node_Name

    # -------- testing ---------
    testn = doingtesting(testing, testn)
    if testn == None:
        returnnone()
    # -------- testing ---------

    if debugsave:
        idf.savecopy("hhh8.idf")

    # # gather all renamed nodes
    # # do the renaming
    renamenodes(idf, 'node')
    # -------- testing ---------
    testn = doingtesting(testing, testn)
    if testn == None:
        returnnone()
    # -------- testing ---------
    if debugsave:
        idf.savecopy("hhh9.idf")
    return thebranch

def main():
    """the main routine"""
    from six import StringIO
    import eppy.iddv7 as iddv7
    IDF.setiddname(StringIO(iddv7.iddtxt))
    idf1 = IDF(StringIO(''))
    loopname = "p_loop"
    sloop = ['sb0', ['sb1', 'sb2', 'sb3'], 'sb4']
    dloop = ['db0', ['db1', 'db2', 'db3'], 'db4']
    # makeplantloop(idf1, loopname, sloop, dloop)
    loopname = "c_loop"
    sloop = ['sb0', ['sb1', 'sb2', 'sb3'], 'sb4']
    dloop = ['db0', ['db1', 'db2', 'db3'], 'db4']
    # makecondenserloop(idf1, loopname, sloop, dloop)
    loopname = "a_loop"
    sloop = ['sb0', ['sb1', 'sb2', 'sb3'], 'sb4']
    dloop = ['zone1', 'zone2', 'zone3']
    makeairloop(idf1, loopname, sloop, dloop)
    idf1.savecopy("hh1.idf")


if __name__ == '__main__':
    main()



# Copyright (c) 2016 Santosh Philip
# =======================================================================
#  Distributed under the MIT License.
#  (See accompanying file LICENSE or copy at
#  http://opensource.org/licenses/MIT)
# =======================================================================

"""functions to use json to modify an idf file"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

def key2elements(key):
    """split key to elements"""
    # words = key.split('.')
    # if len(words) == 4:
    #     return words
    # # there is a dot in object name
    # fieldword = words.pop(-1)
    # nameword = '.'.join(words[-2:])
    # if nameword[-1] in ('"', "'"):
    #     # The object name is in quotes
    #     nameword = nameword[1:-1]
    # elements = words[:-2] + [nameword, fieldword, ]
    # return elements
    words = key.split('.')
    first2words = words[:2]
    lastword = words[-1]
    namewords = words[2:-1]
    namephrase = '.'.join(namewords)
    if namephrase.startswith("'") and namephrase.endswith("'"):
        namephrase = namephrase[1:-1]
    return first2words + [namephrase] + [lastword]
    
    
def updateidf(idf, dct):
    """update idf using dct"""
    for key in list(dct.keys()):
        if key.startswith('idf.'):
            idftag, objkey, objname, field = key2elements(key)
            if objname == '':
                try:
                    idfobj = idf.idfobjects[objkey.upper()][0]
                except IndexError as e:
                    idfobj = idf.newidfobject(objkey.upper())
            else:
                idfobj = idf.getobject(objkey.upper(), objname)
                if idfobj == None:
                    idfobj = idf.newidfobject(objkey.upper(), Name=objname)
            idfobj[field] = dct[key]


# Copyright (c) 2012 Tuan Tran
# =======================================================================
#  Distributed under the MIT License.
#  (See accompanying file LICENSE or copy at
#  http://opensource.org/licenses/MIT)
# =======================================================================

""""This module is used for calculation of zone volume for E+ surfaces"""
# Wrote by Tuan Tran trantuan@hawaii.edu / tranhuuanhtuan@gmail.com
# School of Architecture, University of Hawaii at Manoa

# Find the intersection between two lines
# V = (1/6)*|(a-d).((b-d)x(c-d))| http://en.wikipedia.org/wiki/Tetrahedron
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

try:
    import numpy as np
except ImportError as err:
    from tinynumpy import tinynumpy as np

def vol_tehrahedron(poly):
    """volume of a irregular tetrahedron"""
    p_a = np.array(poly[0])
    p_b = np.array(poly[1])
    p_c = np.array(poly[2])
    p_d = np.array(poly[3])
    return abs(np.dot(
        np.subtract(p_a, p_d),
        np.cross(
            np.subtract(p_b, p_d),
            np.subtract(p_c, p_d))) / 6)

def central_p(poly1, poly2):
    central_point = np.array([0.0, 0.0, 0.0])
    for i in range(len(poly1)):
        central_point = np.add(central_point, np.add(np.array(poly1[i]), np.array(poly2[i])))
    return np.divide(np.divide(central_point, (len(poly1))), 2)

def vol(poly1, poly2):
    """"volume of a zone defined by two polygon bases """
    c_point = central_p(poly1, poly2)
    c_point = (c_point[0], c_point[1], c_point[2])
    vol_therah = 0
    num = len(poly1)
    poly1.append(poly1[0])
    poly2.append(poly2[0])
    for i in range(num - 2):
        # the upper part
        tehrahedron = [c_point, poly1[0], poly1[i+1], poly1[i+2]]
        vol_therah += vol_tehrahedron(tehrahedron)
        # the bottom part
        tehrahedron = [c_point, poly2[0], poly2[i+1], poly2[i+2]]
        vol_therah += vol_tehrahedron(tehrahedron)
    # the middle part
    for i in range(num):
        tehrahedron = [c_point, poly1[i], poly2[i], poly2[i+1]]
        vol_therah += vol_tehrahedron(tehrahedron)
        tehrahedron = [c_point, poly1[i], poly1[i+1], poly2[i]]
        vol_therah += vol_tehrahedron(tehrahedron)
    return vol_therah



# Copyright (c) 2012 Tuan Tran

# This file is part of eppy.
# =======================================================================
#  Distributed under the MIT License.
#  (See accompanying file LICENSE or copy at
#  http://opensource.org/licenses/MIT)
# =======================================================================

"""This module is used for assisted calculations on E+ surfaces"""
# Wrote by Tuan Tran trantuan@hawaii.edu / tranhuuanhtuan@gmail.com
# School of Architecture, University of Hawaii at Manoa

# The following code within the block
# credited by ActiveState Code Recipes code.activestate.com
## {{{ http://code.activestate.com/recipes/578276/ (r1)
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

try:
    import numpy as np
    from numpy import arccos as acos
except ImportError as err:
    from tinynumpy import tinynumpy as np
    from tinynumpy import tinylinalg as linalg
    from math import acos as acos
import math


def area(poly):
    """Area of a polygon poly"""
    if len(poly) < 3: # not a plane - no area
        return 0
    total = [0, 0, 0]
    num = len(poly)
    for i in range(num):
        vi1 = poly[i]
        vi2 = poly[(i+1) % num]
        prod = np.cross(vi1, vi2)
        total[0] += prod[0]
        total[1] += prod[1]
        total[2] += prod[2]
    if total == [0, 0, 0]:  # points are in a straight line - no area
        return 0
    result = np.dot(total, unit_normal(poly[0], poly[1], poly[2]))
    return abs(result/2)

def unit_normal(pt_a, pt_b, pt_c):
    """unit normal vector of plane defined by points pt_a, pt_b, and pt_c"""
    x_val = np.linalg.det([[1, pt_a[1], pt_a[2]], [1, pt_b[1], pt_b[2]], [1, pt_c[1], pt_c[2]]])
    y_val = np.linalg.det([[pt_a[0], 1, pt_a[2]], [pt_b[0], 1, pt_b[2]], [pt_c[0], 1, pt_c[2]]])
    z_val = np.linalg.det([[pt_a[0], pt_a[1], 1], [pt_b[0], pt_b[1], 1], [pt_c[0], pt_c[1], 1]])
    magnitude = (x_val**2 + y_val**2 + z_val**2)**.5
    mag = (x_val/magnitude, y_val/magnitude, z_val/magnitude)
    if magnitude < 0.00000001:
        mag = (0, 0, 0)
    return mag
## end of http://code.activestate.com/recipes/578276/ }}}

# distance between two points
def dist(pt1, pt2):
    """Distance between two points"""
    return ((pt2[0] - pt1[0])**2 + (pt2[1] - pt1[1])**2 + (pt2[2] - pt1[2])**2)**0.5

# width of a rectangular polygon
def width(poly):
    """Width of a polygon poly"""
    num = len(poly) - 1
    if abs(poly[num][2] - poly[0][2]) < abs(poly[1][2] - poly[0][2]):
        return dist(poly[num], poly[0])
    elif abs(poly[num][2] - poly[0][2]) > abs(poly[1][2] - poly[0][2]):
        return dist(poly[1], poly[0])
    else: return max(dist(poly[num], poly[0]), dist(poly[1], poly[0]))

# height of a polygon poly
def height(poly):
    """Height of a polygon poly"""
    num = len(poly) - 1
    if abs(poly[num][2] - poly[0][2]) > abs(poly[1][2] - poly[0][2]):
        return dist(poly[num], poly[0])
    elif abs(poly[num][2] - poly[0][2]) < abs(poly[1][2] - poly[0][2]):
        return dist(poly[1], poly[0])
    else:
        return min(dist(poly[num], poly[0]), dist(poly[1], poly[0]))


def angle2vecs(vec1, vec2):
    """angle between two vectors"""
    # vector a * vector b = |a|*|b|* cos(angle between vector a and vector b)
    dot = np.dot(vec1, vec2)
    vec1_modulus = np.sqrt(np.multiply(vec1, vec1).sum())
    vec2_modulus = np.sqrt(np.multiply(vec2, vec2).sum())
    if (vec1_modulus * vec2_modulus) == 0:
        cos_angle = 1
    else: cos_angle = dot / (vec1_modulus * vec2_modulus)
    return math.degrees(acos(cos_angle))

# orienation of a polygon poly
def azimuth(poly):
    """Azimuth of a polygon poly"""
    num = len(poly) - 1
    vec = unit_normal(poly[0], poly[1], poly[num])
    vec_azi = np.array([vec[0], vec[1], 0])
    vec_n = np.array([0, 1, 0])
    # update by Santosh
    # angle2vecs gives the smallest angle between the vectors
    # so for a west wall angle2vecs will give 90
    # the following 'if' statement will make sure 270 is returned
    x_vector = vec_azi[0]
    if x_vector < 0:
        return 360 - angle2vecs(vec_azi, vec_n)
    else:
        return angle2vecs(vec_azi, vec_n)

def tilt(poly):
    """Tilt of a polygon poly"""
    num = len(poly) - 1
    vec = unit_normal(poly[0], poly[1], poly[num])
    vec_alt = np.array([vec[0], vec[1], vec[2]])
    vec_z = np.array([0, 0, 1])
    # return (90 - angle2vecs(vec_alt, vec_z)) # update by Santosh
    return angle2vecs(vec_alt, vec_z)



# Copyright (c) 2012 Santosh Philip
# =======================================================================
#  Distributed under the MIT License.
#  (See accompanying file LICENSE or copy at
#  http://opensource.org/licenses/MIT)
# =======================================================================

"""images for documentation"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from IPython.display import Image
from IPython.display import display_pretty, display_html, display_jpeg, display_png, display_json, display_latex, display_svg
filemerge = Image(filename='../../docs/images/filemerge.png')
plantloop = Image(filename='../../docs/images/plantloop.png')
idfdiff_path = '../../docs/images/idfdiff.html'



# Copyright (c) 2012 Santosh Philip
# =======================================================================
#  Distributed under the MIT License.
#  (See accompanying file LICENSE or copy at
#  http://opensource.org/licenses/MIT)
# =======================================================================
"""change the edges in loopdaigram so that there are no names with colons (:) """
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

def replace_colon(s, replacewith='__'):
    """replace the colon with something"""
    return s.replace(":", replacewith)
    
def clean_edges(arg):
    if isinstance(arg, str): # Python 3: isinstance(arg, str)
        return replace_colon(arg)
    try:
        return tuple(clean_edges(x) for x in arg)
    except TypeError: # catch when for loop fails
        return replace_colon(arg) # not a sequence so just return repr

        # start pytests +++++++++++++++++++++++

def test_replace_colon():
    """py.test for replace_colon"""
    data = (("zone:aap", '@', "zone@aap"),# s, r, replaced
    )    
    for s, r, replaced in data:
        result = replace_colon(s, r)
        assert result == replaced
        
def test_cleanedges():
    """py.test for cleanedges"""
    data = (([('a:a', 'a'), (('a', 'a'), 'a:a'), ('a:a', ('a', 'a'))],
    (('a__a', 'a'), (('a', 'a'), 'a__a'), ('a__a', ('a', 'a')))), 
    # edg, clean_edg
    )
    for edg, clean_edg in data:
        result = clean_edges(edg)
        assert result == clean_edg
        
# end pytests +++++++++++++++++++++++
        


# Copyright (c) 2012 Santosh Philip
# =======================================================================
#  Distributed under the MIT License.
#  (See accompanying file LICENSE or copy at
#  http://opensource.org/licenses/MIT)
# =======================================================================

"""some helper files"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from string import ascii_letters, digits

def onlylegalchar(name):
    """return only legal chars"""
    legalchar = ascii_letters + digits + ' '
    return ''.join([s for s in name[:] if s in legalchar])

def makefieldname(namefromidd):
    """made a field name that can be used by bunch"""
    newname = onlylegalchar(namefromidd)
    bunchname = newname.replace(' ', '_')
    return bunchname

def matchfieldnames(field_a, field_b):
    """Check match between two strings, ignoring case and spaces/underscores.
    
    Parameters
    ----------
    a : str
    b : str
    
    Returns
    -------
    bool
    
    """
    normalised_a = field_a.replace(' ', '_').lower()
    normalised_b = field_b.replace(' ', '_').lower()
    
    return normalised_a == normalised_b

def intinlist(lst):
    """test if int in list"""
    for item in lst:
        try:
            item = int(item)
            return True
        except ValueError:
            pass
    return False

def replaceint(fname, replacewith='%s'):
    """replace int in lst"""
    words = fname.split()
    for i, word in enumerate(words):
        try:
            word = int(word)
            words[i] = replacewith
        except ValueError:
            pass
    return ' '.join(words)

def cleaniddfield(acomm):
    """make all the keys lower case"""
    for key in list(acomm.keys()):
        val = acomm[key]
        acomm[key.lower()] = val
    for key in list(acomm.keys()):
        val = acomm[key]
        if key != key.lower():
            acomm.pop(key)
    return acomm

def cleancommdct(commdct):
    """make all keys in commdct lower case"""
    return [[cleaniddfield(fcomm) for fcomm in comm] for comm in commdct]
    



from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

# coding: utf-8

# In[1]:

import sys
# pathnameto_eppy = 'c:/eppy'
pathnameto_eppy = '../../'
sys.path.append(pathnameto_eppy)

from eppy import modeleditor
from eppy.modeleditor import IDF
iddfile = "../../eppy/resources/iddfiles/Energy+V8_1_0.idd"
fname1 = "../../eppy/resources/idffiles/V8_1_0/Boiler.idf"


# In[2]:

IDF.setiddname(iddfile)
idf1 = IDF(fname1)


# In[3]:

idf1 = IDF(fname1)


# In[4]:

idf1.saveas("bl_original.idf")


# In[5]:

keys = list(idf1.idfobjects.keys())


# In[6]:

# rename object name with a colon and any field that refers to that name
from eppy.bunch_subclass import BadEPFieldError
renamednames = []
for key in keys:
    for idfobject in idf1.idfobjects[key]:
        try:
            name = idfobject.Name
            if name.find(":") != -1:
                renamednames.append(name)
                newname = name.replace(':', '__') 
                # print "%s, %s, %s" %  (key, name, newname)
                modeleditor.rename(idf1, key, name, newname)
        except BadEPFieldError as e:
            pass
        


# In[7]:

idf1.saveas("bl_renamed_names.idf")

# rename any fields that have a colon 
for key in keys:
    for idfobject in idf1.idfobjects[key]:
        objectfields = idfobject['obj']
        # idfobject['obj'] is unocumented functionality 
        # to get fields of an object as a list
        for i, field in enumerate(objectfields[1:]):
            try:
                if field in renamednames:
                    if field.find(":") != -1:
                        newfield = field.replace(":", "__")
                        objectfields[i + 1] = newfield
            except AttributeError as e:
                # may be a number
                pass

idf1.saveas("bl_renamed_fields.idf")

# rename any field that is a node
for key in keys:
    for idfobject in idf1.idfobjects[key]:
        # idfobject['obj'] is unocumented functionality 
        # to get fields of an object as a list
        objectfields = idfobject['obj']
        # another undocumented functionality
        objectidds = idfobject['objidd']
        for i, (field, f_idd) in enumerate(zip(objectfields[1:],
                                            objectidds[1:])):
            try:
                # print f_idd["field"][0]
                nodeinname = "Node" in f_idd["field"][0]
            except KeyError as e:
                # field may not be a key
                nodeinname = False
            if nodeinname:
                try:
                    # if field in renamednames:
                    if field.find(":") != -1:
                        newfield = field.replace(":", "__")
                        objectfields[i + 1] = newfield
                        # print f_idd["field"], field
                except AttributeError as e:
                    # may be a number
                    pass
                
idf1.saveas("bl_renamed_nodes.idf")


# to make the loop diagram do:
# python loopdiagram.py ../../eppy/resources/iddfiles/Energy+V8_1_0.idd bl_renamed_nodes.idf






# Copyright (c) 2016 Santosh Philip
# =======================================================================
#  Distributed under the MIT License.
#  (See accompanying file LICENSE or copy at
#  http://opensource.org/licenses/MIT)
"""extract the groups from the iddfile"""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

def nocomment(astr, com):
    """
    just like the comment in python.
    removes any text after the phrase 'com'
    """
    alist = astr.splitlines()
    for i in range(len(alist)):
        element = alist[i]
        pnt = element.find(com)
        if pnt != -1:
            alist[i] = element[:pnt]
    return '\n'.join(alist)


def idd2group(fhandle):
    """wrapper for iddtxt2groups"""
    try:
        txt = fhandle.read()
        return iddtxt2groups(txt)
    except AttributeError as e:
        txt = open(fhandle, 'r').read()
        return iddtxt2groups(txt)

def idd2grouplist(fhandle):
    """wrapper for iddtxt2grouplist"""
    try:
        txt = fhandle.read()
        return iddtxt2grouplist(txt)
    except AttributeError as e:
        txt = open(fhandle, 'r').read()
        return iddtxt2grouplist(txt)


def iddtxt2groups(txt):
    """extract the groups from the idd file"""
    try:
        txt = txt.decode('ISO-8859-2')
    except AttributeError as e:
        pass # for python 3
    txt = nocomment(txt, '!')
    txt = txt.replace("\\group", "!-group") # retains group in next line
    txt = nocomment(txt, '\\') # remove all other idd info
    lines = txt.splitlines()
    lines = [line.strip() for line in lines] # cleanup
    lines = [line for line in lines if line != ''] # cleanup
    txt = '\n'.join(lines)
    gsplits = txt.split('!') # split into groups, since we have !-group
    gsplits = [gsplit.splitlines() for gsplit in gsplits] # split group

    gsplits[0].insert(0, None)
        # Put None for the first group that does nothave a group name
    gdict = {}
    for gsplit in gsplits:
        gdict.update({gsplit[0]:gsplit[1:]})
        # makes dict {groupname:[k1, k2], groupname2:[k3, k4]}

    gdict = {k:'\n'.join(v) for k, v in gdict.items()}# joins lines back
    gdict = {k:v.split(';') for k, v in gdict.items()} # splits into idfobjects
    gdict = {k:[i.strip() for i in v] for k, v in gdict.items()} # cleanup
    gdict = {k:[i.splitlines() for i in v] for k, v in gdict.items()}
        # splits idfobjects into lines
    gdict = {k:[i for i in v if len(i) > 0] for k, v in gdict.items()}
        # cleanup - removes blank lines
    gdict = {k:[i[0] for i in v] for k, v in gdict.items()} # use first line
    gdict = {k:[i.split(',')[0] for i in v] for k, v in gdict.items()}
        # remove ','
    nvalue = gdict.pop(None) # remove group with no name
    gdict = {k[len('-group '):]:v for k, v in gdict.items()} # get group name
    gdict.update({None:nvalue}) # put back group with no name
    return gdict

def iddtxt2grouplist(txt):
    """return a list of group names
    the list in the same order as the idf objects in idd file
    """
    def makenone(astr):
        if astr == 'None':
            return None
        else:
            return astr

    txt = nocomment(txt, '!')
    txt = txt.replace("\\group", "!-group") # retains group in next line
    txt = nocomment(txt, '\\') # remove all other idd info
    lines = txt.splitlines()
    lines = [line.strip() for line in lines] # cleanup
    lines = [line for line in lines if line != ''] # cleanup
    txt = '\n'.join(lines)
    gsplits = txt.split('!') # split into groups, since we have !-group
    gsplits = [gsplit.splitlines() for gsplit in gsplits] # split group

    gsplits[0].insert(0, u'-group None')
        # Put None for the first group that does nothave a group name
    
    glist = []
    for gsplit in gsplits:
        glist.append((gsplit[0], gsplit[1:]))
        # makes dict {groupname:[k1, k2], groupname2:[k3, k4]}

    glist = [(k, '\n'.join(v)) for k, v in glist]# joins lines back
    glist = [(k, v.split(';')) for k, v in glist] # splits into idfobjects
    glist = [(k, [i.strip() for i in v]) for k, v in glist] # cleanup
    glist = [(k, [i.splitlines() for i in v]) for k, v in glist]
        # splits idfobjects into lines
    glist = [(k, [i for i in v if len(i) > 0]) for k, v in glist]
        # cleanup - removes blank lines
    glist = [(k, [i[0] for i in v]) for k, v in glist] # use first line
    fglist = []
    for gnamelist in glist:
        gname = gnamelist[0]
        thelist = gnamelist[-1]
        for item in thelist:
            fglist.append((gname, item))
    glist = [(gname[len("-group "):], obj) for gname, obj in fglist] # remove "-group "
    glist = [(makenone(gname), obj) for gname, obj in glist] # make str None into None
    glist = [(gname, obj.split(',')[0]) for gname, obj in glist] # remove comma
    return glist

def group2commlst(commlst, glist):
    """add group info to commlst"""
    for (gname, objname), commitem in zip(glist, commlst):
        newitem1 = "group %s" % (gname, )
        newitem2 = "idfobj %s" % (objname, )
        commitem[0].insert(0, newitem1)
        commitem[0].insert(1, newitem2)
    return commlst
    
def group2commdct(commdct, glist):
    """add group info tocomdct"""
    for (gname, objname), commitem in zip(glist, commdct):
        commitem[0]['group'] = gname
        commitem[0]['idfobj'] = objname
    return commdct

def commdct2grouplist(gcommdct):
    """extract embedded group data from commdct.
    return gdict -> {g1:[obj1, obj2, obj3], g2:[obj4, ..]}"""
    gdict = {}
    for objidd in gcommdct:
        group = objidd[0]['group']
        objname = objidd[0]['idfobj']
        if group in gdict:
            gdict[group].append(objname)
        else:
            gdict[group] = [objname, ]
    return gdict



# Copyright (c) 2012 Santosh Philip
# =======================================================================
#  Distributed under the MIT License.
#  (See accompanying file LICENSE or copy at
#  http://opensource.org/licenses/MIT)
# =======================================================================

# along with Eppy.  If not, see <http://www.gnu.org/licenses/>.

"""
Do a diff between two idf files.
Prints the diff in csv  or html file format.
You can redirect the output to a file and open the file using as a spreadsheet or by using a browser
"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import argparse

import sys
import itertools
from bs4 import BeautifulSoup, Tag

pathnameto_eplusscripting = "../../"
sys.path.append(pathnameto_eplusscripting)

from eppy.bunch_subclass import BadEPFieldError
from eppy.modeleditor import IDF



help_message = '''
The help message goes here.
'''


class Usage(Exception):
    def __init__(self, msg):
        self.msg = msg


def getobjname(item):
    """return obj name or blank """
    try:
        objname = item.Name
    except BadEPFieldError as e:
        objname = ' '
    return objname


def theheader(n1, n2):
    """return the csv header"""
    s = "Object Key, Object Name, Field Name, %s, %s" % ('file1', 'file2')
    return s.split(',')

class DtlsSorter(object):
    """helps me to sort it using the order of keys in idd file"""
    def __init__(self, dtls):
        self.dtlsorder = {j:i for i, j in enumerate(dtls)}
    def getkey(self, item):
        return self.dtlsorder[item[0]] # item[0] is the object key

def makecsvdiffs(thediffs, dtls, n1, n2):
    """return the csv to be displayed"""
    def ishere(val):
        if val == None:
            return "not here"
        else:
            return "is here"
    rows = []
    rows.append(['file1 = %s' % (n1, )])
    rows.append(['file2 = %s' % (n2, )])
    rows.append('')
    rows.append(theheader(n1, n2))
    keys = list(thediffs.keys()) # ensures sorting by Name
    keys.sort()
    # sort the keys in the same order as in the idd
    dtlssorter = DtlsSorter(dtls)
    keys = sorted(keys, key=dtlssorter.getkey)
    for key in keys:
        if len(key) == 2:
            rw2 = [''] + [ishere(i) for i in thediffs[key]]
        else:
            rw2 = list(thediffs[key])
        rw1 = list(key)
        rows.append(rw1 + rw2)
    return rows


def idfdiffs(idf1, idf2):
    """return the diffs between the two idfs"""
    thediffs = {}
    keys = idf1.model.dtls # undocumented variable

    for akey in keys:
        idfobjs1 = idf1.idfobjects[akey]
        idfobjs2 = idf2.idfobjects[akey]
        names = set([getobjname(i) for i in idfobjs1] +
                    [getobjname(i) for i in idfobjs2])
        names = sorted(names)
        idfobjs1 = sorted(idfobjs1, key=lambda idfobj: idfobj['obj'])
        idfobjs2 = sorted(idfobjs2, key=lambda idfobj: idfobj['obj'])
        for name in names:
            n_idfobjs1 = [item for item in idfobjs1
                          if getobjname(item) == name]
            n_idfobjs2 = [item for item in idfobjs2
                          if getobjname(item) == name]
            for idfobj1, idfobj2 in itertools.zip_longest(n_idfobjs1,
                                                           n_idfobjs2):
                if idfobj1 == None:
                    thediffs[(idfobj2.key.upper(), 
                                getobjname(idfobj2))] = (None, idf1.idfname) #(idf1.idfname, None) -> old
                    break
                if idfobj2 == None:
                    thediffs[(idfobj1.key.upper(), 
                                getobjname(idfobj1))] = (idf2.idfname, None) # (None, idf2.idfname) -> old
                    break
                for i, (f1, f2) in enumerate(zip(idfobj1.obj, idfobj2.obj)):
                    if i == 0:
                        f1, f2 = f1.upper(), f2.upper()
                    if f1 != f2:
                        thediffs[(
                            akey,
                            getobjname(idfobj1),
                            idfobj1.objidd[i]['field'][0])] = (f1, f2)
    return thediffs

def printcsv(csvdiffs):
    """print the csv"""
    for row in csvdiffs:
        print(','.join([str(cell) for cell in row]))

def heading2table(soup, table, row):
    """add heading row to table"""
    tr = Tag(soup, name="tr")
    table.append(tr)
    for attr in row:
        th = Tag(soup, name="th")
        tr.append(th)
        th.append(attr)

def row2table(soup, table, row):
    """ad a row to the table"""
    tr = Tag(soup, name="tr")
    table.append(tr)
    for attr in row:
        td = Tag(soup, name="td")
        tr.append(td)
        td.append(attr)


def printhtml(csvdiffs):
    """print the html"""
    soup = BeautifulSoup()
    html = Tag(soup, name="html")
    para1 = Tag(soup, name="p")
    para1.append(csvdiffs[0][0])
    para2 = Tag(soup, name="p")
    para2.append(csvdiffs[1][0])
    table = Tag(soup, name="table")
    table.attrs.update(dict(border="1"))

    soup.append(html)
    html.append(para1)
    html.append(para2)
    html.append(table)
    heading2table(soup, table, csvdiffs[3])
    for row in csvdiffs[4:]:
        row = [str(cell) for cell in row]
        row2table(soup, table, row)
    # print soup.prettify()
    print(soup)

if __name__ == '__main__':
    # do the argparse stuff
    parser = argparse.ArgumentParser(usage=None, description=__doc__)
    parser.add_argument(
        'idd', action='store',
        help='location of idd file = ./somewhere/eplusv8-0-1.idd')
    parser.add_argument(
        'file1', action='store',
        help='location of first with idf files = ./somewhere/f1.idf')
    parser.add_argument(
        'file2', action='store',
        help='location of second with idf files = ./somewhere/f2.idf')
    group = parser.add_mutually_exclusive_group(required=True)
    group.add_argument('--csv', action='store_true')
    group.add_argument('--html', action='store_true')
    nspace = parser.parse_args()
    fname1 = nspace.file1
    fname2 = nspace.file2
    iddfile = nspace.idd
    IDF.setiddname(iddfile)
    idf1 = IDF(fname1)
    idf2 = IDF(fname2)
    dtls = idf1.model.dtls # undocumented variable
    thediffs = idfdiffs(idf1, idf2)
    csvdiffs = makecsvdiffs(thediffs, dtls, idf1.idfname, idf2.idfname)
    if nspace.csv:
        printcsv(csvdiffs)
    elif nspace.html:
        printhtml(csvdiffs)


# python idfdiff.py --csv ../resources/iddfiles/Energy+V7_2_0.idd ../resources/idffiles/V_7_2/constr.idf ../resources/idffiles/V_7_2/constr_diff.idf
# python idfdiff.py --html ../resources/iddfiles/Energy+V7_2_0.idd ../resources/idffiles/V_7_2/constr.idf ../resources/idffiles/V_7_2/constr_diff.idf
# python idfdiff2.py --html ../resources/iddfiles/Energy+V8_0_0.idd ../resources/idffiles/V8_0_0/5ZoneSupRetPlenRAB.idf ../resources/idffiles/V8_0_0/5ZoneWaterLoopHeatPump.idf


# Copyright (c) 2012 Tuan Tran
# =======================================================================
#  Distributed under the MIT License.
#  (See accompanying file LICENSE or copy at
#  http://opensource.org/licenses/MIT)
# =======================================================================

## {{{ http://code.activestate.com/recipes/578276/ (r1)
## modified by Tuan Tran trantuan@hawaii.edu at L+U, www.coolshadow.com
"""height and surface"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

try:
    import numpy as np
except ImportError as err:
    import tinynumpy as np


# area of a polygon
def area(poly):
    """area"""
    if len(poly) < 3: # not a plane - no area
        return 0
    total = [0, 0, 0]
    num = len(poly)
    for i in range(num):
        vi1 = poly[i]
        vi2 = poly[(i+1) % num]
        prod = np.cross(vi1, vi2)
        total[0] += prod[0]
        total[1] += prod[1]
        total[2] += prod[2]
    result = np.dot(total, unit_normal(poly[0], poly[1], poly[2]))
    return abs(result/2)

# average height of a polygon
def height(poly):
    """height"""
    num = len(poly)
    hgt = 0.0
    for i in range(num):
        hgt += (poly[i][2])
    return hgt/num

#unit normal vector of plane defined by points a, b, and c
def unit_normal(apnt, bpnt, cpnt):
    """unit normal"""
    xvar = np.tinylinalg.det([
        [1, apnt[1], apnt[2]], [1, bpnt[1], bpnt[2]], [1, cpnt[1], cpnt[2]]])
    yvar = np.tinylinalg.det([
        [apnt[0], 1, apnt[2]], [bpnt[0], 1, bpnt[2]], [cpnt[0], 1, cpnt[2]]])
    zvar = np.tinylinalg.det([
        [apnt[0], apnt[1], 1], [bpnt[0], bpnt[1], 1], [cpnt[0], cpnt[1], 1]])
    magnitude = (xvar**2 + yvar**2 + zvar**2)**.5
    if magnitude < 0.00000001:
        mag = (0, 0, 0)
    else: mag = (xvar/magnitude, yvar/magnitude, zvar/magnitude)
    return mag
## end of http://code.activestate.com/recipes/578276/ }}}




# Copyright (c) 2016 Jamie Bull
# =======================================================================
#  Distributed under the MIT License.
#  (See accompanying file LICENSE or copy at
#  http://opensource.org/licenses/MIT)
# =======================================================================
"""Run functions for EnergyPlus.
"""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import os
import platform
import pydoc
import shutil
from subprocess import CalledProcessError
from subprocess import check_call
import tempfile

try:
    import multiprocessing as mp
except ImportError:
    pass


def install_paths(version):
    """Get the install paths for EnergyPlus executable and weather files.
    """
    if platform.system() == 'Windows':
        eplus_home = "C:/EnergyPlusV{version}".format(**locals())
        eplus_exe = os.path.join(eplus_home, 'energyplus.exe')
    elif platform.system() == "Linux":
        eplus_home = "/usr/local/EnergyPlus-{version}".format(**locals())
        eplus_exe = os.path.join(eplus_home, 'energyplus')
    else:
        eplus_home = "/Applications/EnergyPlus-{version}".format(**locals())
        eplus_exe = os.path.join(eplus_home, 'energyplus')

    eplus_weather = os.path.join(eplus_home, 'WeatherData')

    return eplus_exe, eplus_weather


def wrapped_help_text(wrapped_func):
    """Decorator to pass through the documentation from a wrapped function.
    """
    def decorator(wrapper_func):
        """The decorator.

        Parameters
        ----------
        f : callable
            The wrapped function.

        """
        wrapper_func.__doc__ = ('This method wraps the following method:\n\n' +
                                pydoc.text.document(wrapped_func))
        return wrapper_func
    return decorator


def runIDFs(jobs_list, processors=1):
    """Wrapper for run() to be used when running IDF5 runs in parallel.

    Parameters
    ----------
    jobs_list : list
        A list made up of an IDF5 object and a kwargs dict (see .

    processors : int, optional
        Number of processors to run on (default: 1). If 0 is passed then
        the process will run on all CPUs, -1 means one less than all CPUs, etc.

    """
    if processors <= 0:
        processors = max(1, mp.cpu_count() - processors)

    shutil.rmtree("multi_runs", ignore_errors=True)
    os.mkdir("multi_runs")

    processed_runs = []
    for i, item in enumerate(jobs_list):
        idf = item[0]
        epw = idf.epw
        kwargs = item[1]
        idf_dir = os.path.join('multi_runs', 'idf_%i' % i)
        os.mkdir(idf_dir)
        idf_path = os.path.join(idf_dir, 'in.idf')
        idf.saveas(idf_path)
        processed_runs.append([[idf_path, epw], kwargs])

    try:
        pool = mp.Pool(processors)
        pool.map(multirunner, processed_runs)
        pool.close()
    except NameError:
        # multiprocessing not present so pass the jobs one at a time
        for job in processed_runs:
            multirunner([job])
    shutil.rmtree("multi_runs", ignore_errors=True)


def multirunner(args):
    """Wrapper for run() to be used when running IDF and EPW runs in parallel.

    Parameters
    ----------
    args : list
        A list made up of a two-item list (IDF and EPW) and a kwargs dict.

    """
    run(*args[0], **args[1])


def run(idf=None, weather=None, output_directory='', annual=False,
        design_day=False, idd=None, epmacro=False, expandobjects=False,
        readvars=False, output_prefix=None, output_suffix=None, version=False,
        verbose='v', ep_version=None):
    """
    Wrapper around the EnergyPlus command line interface.

    Parameters
    ----------
    idf : str
        Full or relative path to the IDF file to be run, or an IDF object.

    weather : str
        Full or relative path to the weather file.

    output_directory : str, optional
        Full or relative path to an output directory (default: 'run_outputs)

    annual : bool, optional
        If True then force annual simulation (default: False)

    design_day : bool, optional
        Force design-day-only simulation (default: False)

    idd : str, optional
        Input data dictionary (default: Energy+.idd in EnergyPlus directory)

    epmacro : str, optional
        Run EPMacro prior to simulation (default: False).

    expandobjects : bool, optional
        Run ExpandObjects prior to simulation (default: False)

    readvars : bool, optional
        Run ReadVarsESO after simulation (default: False)

    output_prefix : str, optional
        Prefix for output file names (default: eplus)

    output_suffix : str, optional
        Suffix style for output file names (default: L)
            L: Legacy (e.g., eplustbl.csv)
            C: Capital (e.g., eplusTable.csv)
            D: Dash (e.g., eplus-table.csv)

    version : bool, optional
        Display version information (default: False)

    verbose: str
        Set verbosity of runtime messages (default: v)
            v: verbose
            q: quiet

    ep_version: str
        EnergyPlus version, used to find install directory. Required if run() is
        called with an IDF file path rather than an IDF object.

    Returns
    -------
    str : status

    Raises
    ------
    CalledProcessError

    AttributeError
        If no ep_version parameter is passed when calling with an IDF file path
        rather than an IDF object.


    """
    args = locals().copy()
    # get unneeded params out of args ready to pass the rest to energyplus.exe
    verbose = args.pop('verbose')
    idf = args.pop('idf')
    try:
        idf_path = os.path.abspath(idf.idfname)
    except AttributeError:
        idf_path = os.path.abspath(idf)
    ep_version = args.pop('ep_version')
    # get version from IDF object or by parsing the IDF file for it
    if not ep_version:
        try:
            ep_version = '-'.join(str(x) for x in idf.idd_version[:3])
        except AttributeError:
            raise AttributeError(
                "The ep_version must be set when passing an IDF path. \
                Alternatively, use IDF.run()")
    eplus_exe_path, eplus_weather_path = install_paths(ep_version)
    if version:
        # just get EnergyPlus version number and return
        cmd = [eplus_exe_path, '--version']
        check_call(cmd)
        return

    # convert paths to absolute paths if required
    if os.path.isfile(args['weather']):
        args['weather'] = os.path.abspath(args['weather'])
    else:
        args['weather'] = os.path.join(eplus_weather_path, args['weather'])
    args['output_directory'] = os.path.abspath(args['output_directory'])

    # store the directory we start in
    cwd = os.getcwd()
    run_dir = os.path.abspath(tempfile.mkdtemp())
    os.chdir(run_dir)

    # build a list of command line arguments
    cmd = [eplus_exe_path]
    for arg in args:
        if args[arg]:
            if isinstance(args[arg], bool):
                args[arg] = ''
            cmd.extend(['--{}'.format(arg.replace('_', '-'))])
            if args[arg] != "":
                cmd.extend([args[arg]])
    cmd.extend([idf_path])

    try:
        if verbose == 'v':
            check_call(cmd)
        elif verbose == 'q':
            check_call(cmd, stdout=open(os.devnull, 'w'))
        os.chdir(cwd)
    except CalledProcessError:
        # potentially catch contents of std out and put it in the error
        raise
    return 'OK'



from eppy.iddcurrent import iddcurrent
from eppy.modeleditor import IDF
from six import StringIO


iddsnippet = iddcurrent.iddtxt
iddfhandle = StringIO(iddcurrent.iddtxt)
IDF.setiddname(iddfhandle)

idftxt = """

Version,                  
    8.5;                      !- Version Identifier

Building,                 
    Building 1,               !- Name
    ,                         !- North Axis
    ,                         !- Terrain
    ,                         !- Loads Convergence Tolerance Value
    ,                         !- Temperature Convergence Tolerance Value
    ,                         !- Solar Distribution
    ,                         !- Maximum Number of Warmup Days
    ;                         !- Minimum Number of Warmup Days

Zone,                     
    Thermal Zone 1,           !- Name
    -0.0,                     !- Direction of Relative North
    3.41258124196863,         !- X Origin
    0.821279819391803,        !- Y Origin
    0.7279,                   !- Z Origin
    ,                         !- Type
    ,                         !- Multiplier
    ,                         !- Ceiling Height
    ,                         !- Volume
    ,                         !- Floor Area
    ,                         !- Zone Inside Convection Algorithm
    ;                         !- Zone Outside Convection Algorithm

Zone,                     
    Thermal Zone 2,           !- Name
    -0.0,                     !- Direction of Relative North
    3.41258124196863,         !- X Origin
    0.821279819391803,        !- Y Origin
    0.0,                      !- Z Origin
    ,                         !- Type
    ,                         !- Multiplier
    ,                         !- Ceiling Height
    ,                         !- Volume
    ,                         !- Floor Area
    ,                         !- Zone Inside Convection Algorithm
    ;                         !- Zone Outside Convection Algorithm

BuildingSurface:Detailed, 
    z1 Floor 0001,            !- Name
    Floor,                    !- Surface Type
    ,                         !- Construction Name
    Thermal Zone 1,           !- Zone Name
    Ground,                   !- Outside Boundary Condition
    ,                         !- Outside Boundary Condition Object
    NoSun,                    !- Sun Exposure
    NoWind,                   !- Wind Exposure
    ,                         !- View Factor to Ground
    ,                         !- Number of Vertices
    -0.259,                   !- Vertex 1 Xcoordinate
    2.46,                     !- Vertex 1 Ycoordinate
    0.0,                      !- Vertex 1 Zcoordinate
    -0.259,                   !- Vertex 2 Xcoordinate
    0.4,                      !- Vertex 2 Ycoordinate
    0.0,                      !- Vertex 2 Zcoordinate
    -1.68,                    !- Vertex 3 Xcoordinate
    0.4,                      !- Vertex 3 Ycoordinate
    0.0,                      !- Vertex 3 Zcoordinate
    -1.68,                    !- Vertex 4 Xcoordinate
    2.46,                     !- Vertex 4 Ycoordinate
    0.0;                      !- Vertex 4 Zcoordinate

BuildingSurface:Detailed, 
    z1 Wall 0001,             !- Name
    Wall,                     !- Surface Type
    ,                         !- Construction Name
    Thermal Zone 1,           !- Zone Name
    Outdoors,                 !- Outside Boundary Condition
    ,                         !- Outside Boundary Condition Object
    SunExposed,               !- Sun Exposure
    WindExposed,              !- Wind Exposure
    ,                         !- View Factor to Ground
    ,                         !- Number of Vertices
    -0.259,                   !- Vertex 1 Xcoordinate
    2.46,                     !- Vertex 1 Ycoordinate
    0.7279,                   !- Vertex 1 Zcoordinate
    -0.259,                   !- Vertex 2 Xcoordinate
    2.46,                     !- Vertex 2 Ycoordinate
    0.0,                      !- Vertex 2 Zcoordinate
    -1.68,                    !- Vertex 3 Xcoordinate
    2.46,                     !- Vertex 3 Ycoordinate
    0.0,                      !- Vertex 3 Zcoordinate
    -1.68,                    !- Vertex 4 Xcoordinate
    2.46,                     !- Vertex 4 Ycoordinate
    0.7279;                   !- Vertex 4 Zcoordinate

BuildingSurface:Detailed, 
    z1 Wall 0002,             !- Name
    Wall,                     !- Surface Type
    ,                         !- Construction Name
    Thermal Zone 1,           !- Zone Name
    Outdoors,                 !- Outside Boundary Condition
    ,                         !- Outside Boundary Condition Object
    SunExposed,               !- Sun Exposure
    WindExposed,              !- Wind Exposure
    ,                         !- View Factor to Ground
    ,                         !- Number of Vertices
    -0.259,                   !- Vertex 1 Xcoordinate
    0.4,                      !- Vertex 1 Ycoordinate
    0.7279,                   !- Vertex 1 Zcoordinate
    -0.259,                   !- Vertex 2 Xcoordinate
    0.4,                      !- Vertex 2 Ycoordinate
    0.0,                      !- Vertex 2 Zcoordinate
    -0.259,                   !- Vertex 3 Xcoordinate
    2.46,                     !- Vertex 3 Ycoordinate
    0.0,                      !- Vertex 3 Zcoordinate
    -0.259,                   !- Vertex 4 Xcoordinate
    2.46,                     !- Vertex 4 Ycoordinate
    0.7279;                   !- Vertex 4 Zcoordinate

BuildingSurface:Detailed, 
    z1 Wall 0003,             !- Name
    Wall,                     !- Surface Type
    ,                         !- Construction Name
    Thermal Zone 1,           !- Zone Name
    Outdoors,                 !- Outside Boundary Condition
    ,                         !- Outside Boundary Condition Object
    SunExposed,               !- Sun Exposure
    WindExposed,              !- Wind Exposure
    ,                         !- View Factor to Ground
    ,                         !- Number of Vertices
    -1.68,                    !- Vertex 1 Xcoordinate
    0.4,                      !- Vertex 1 Ycoordinate
    0.7279,                   !- Vertex 1 Zcoordinate
    -1.68,                    !- Vertex 2 Xcoordinate
    0.4,                      !- Vertex 2 Ycoordinate
    0.0,                      !- Vertex 2 Zcoordinate
    -0.259,                   !- Vertex 3 Xcoordinate
    0.4,                      !- Vertex 3 Ycoordinate
    0.0,                      !- Vertex 3 Zcoordinate
    -0.259,                   !- Vertex 4 Xcoordinate
    0.4,                      !- Vertex 4 Ycoordinate
    0.7279;                   !- Vertex 4 Zcoordinate

BuildingSurface:Detailed, 
    z1 Wall 0004,             !- Name
    Wall,                     !- Surface Type
    ,                         !- Construction Name
    Thermal Zone 1,           !- Zone Name
    Outdoors,                 !- Outside Boundary Condition
    ,                         !- Outside Boundary Condition Object
    SunExposed,               !- Sun Exposure
    WindExposed,              !- Wind Exposure
    ,                         !- View Factor to Ground
    ,                         !- Number of Vertices
    -1.68,                    !- Vertex 1 Xcoordinate
    2.46,                     !- Vertex 1 Ycoordinate
    0.7279,                   !- Vertex 1 Zcoordinate
    -1.68,                    !- Vertex 2 Xcoordinate
    2.46,                     !- Vertex 2 Ycoordinate
    0.0,                      !- Vertex 2 Zcoordinate
    -1.68,                    !- Vertex 3 Xcoordinate
    0.4,                      !- Vertex 3 Ycoordinate
    0.0,                      !- Vertex 3 Zcoordinate
    -1.68,                    !- Vertex 4 Xcoordinate
    0.4,                      !- Vertex 4 Ycoordinate
    0.7279;                   !- Vertex 4 Zcoordinate

BuildingSurface:Detailed, 
    z1 Roof 0001,             !- Name
    Roof,                     !- Surface Type
    ,                         !- Construction Name
    Thermal Zone 1,           !- Zone Name
    Outdoors,                 !- Outside Boundary Condition
    ,                         !- Outside Boundary Condition Object
    SunExposed,               !- Sun Exposure
    WindExposed,              !- Wind Exposure
    ,                         !- View Factor to Ground
    ,                         !- Number of Vertices
    -0.259,                   !- Vertex 1 Xcoordinate
    0.4,                      !- Vertex 1 Ycoordinate
    0.7279,                   !- Vertex 1 Zcoordinate
    -0.259,                   !- Vertex 2 Xcoordinate
    2.46,                     !- Vertex 2 Ycoordinate
    0.7279,                   !- Vertex 2 Zcoordinate
    -1.68,                    !- Vertex 3 Xcoordinate
    2.46,                     !- Vertex 3 Ycoordinate
    0.7279,                   !- Vertex 3 Zcoordinate
    -1.68,                    !- Vertex 4 Xcoordinate
    0.4,                      !- Vertex 4 Ycoordinate
    0.7279;                   !- Vertex 4 Zcoordinate

BuildingSurface:Detailed, 
    z2 Floor 0001,            !- Name
    Floor,                    !- Surface Type
    ,                         !- Construction Name
    Thermal Zone 2,           !- Zone Name
    Ground,                   !- Outside Boundary Condition
    ,                         !- Outside Boundary Condition Object
    NoSun,                    !- Sun Exposure
    NoWind,                   !- Wind Exposure
    ,                         !- View Factor to Ground
    ,                         !- Number of Vertices
    0.0,                      !- Vertex 1 Xcoordinate
    2.9,                      !- Vertex 1 Ycoordinate
    0.7279,                      !- Vertex 1 Zcoordinate
    0.0,                      !- Vertex 2 Xcoordinate
    0.0,                      !- Vertex 2 Ycoordinate
    0.7279,                      !- Vertex 2 Zcoordinate
    -2.14,                    !- Vertex 3 Xcoordinate
    0.0,                      !- Vertex 3 Ycoordinate
    0.7279,                      !- Vertex 3 Zcoordinate
    -2.14,                    !- Vertex 4 Xcoordinate
    2.9,                      !- Vertex 4 Ycoordinate
    0.7279;                      !- Vertex 4 Zcoordinate

BuildingSurface:Detailed, 
    z2 Wall 0001,             !- Name
    Wall,                     !- Surface Type
    ,                         !- Construction Name
    Thermal Zone 2,           !- Zone Name
    Outdoors,                 !- Outside Boundary Condition
    ,                         !- Outside Boundary Condition Object
    SunExposed,               !- Sun Exposure
    WindExposed,              !- Wind Exposure
    ,                         !- View Factor to Ground
    ,                         !- Number of Vertices
    -2.14,                    !- Vertex 1 Xcoordinate
    0.0,                      !- Vertex 1 Ycoordinate
    1.458,                   !- Vertex 1 Zcoordinate
    -2.14,                    !- Vertex 2 Xcoordinate
    0.0,                      !- Vertex 2 Ycoordinate
    0.7279,                      !- Vertex 2 Zcoordinate
    0.0,                      !- Vertex 3 Xcoordinate
    0.0,                      !- Vertex 3 Ycoordinate
    0.7279,                      !- Vertex 3 Zcoordinate
    0.0,                      !- Vertex 4 Xcoordinate
    0.0,                      !- Vertex 4 Ycoordinate
    1.458;                   !- Vertex 4 Zcoordinate

BuildingSurface:Detailed, 
    z2 Wall 0002,             !- Name
    Wall,                     !- Surface Type
    ,                         !- Construction Name
    Thermal Zone 2,           !- Zone Name
    Outdoors,                 !- Outside Boundary Condition
    ,                         !- Outside Boundary Condition Object
    SunExposed,               !- Sun Exposure
    WindExposed,              !- Wind Exposure
    ,                         !- View Factor to Ground
    ,                         !- Number of Vertices
    -2.14,                    !- Vertex 1 Xcoordinate
    2.9,                      !- Vertex 1 Ycoordinate
    1.458,                   !- Vertex 1 Zcoordinate
    -2.14,                    !- Vertex 2 Xcoordinate
    2.9,                      !- Vertex 2 Ycoordinate
    0.7279,                      !- Vertex 2 Zcoordinate
    -2.14,                    !- Vertex 3 Xcoordinate
    0.0,                      !- Vertex 3 Ycoordinate
    0.7279,                      !- Vertex 3 Zcoordinate
    -2.14,                    !- Vertex 4 Xcoordinate
    0.0,                      !- Vertex 4 Ycoordinate
    1.458;                   !- Vertex 4 Zcoordinate

BuildingSurface:Detailed, 
    z2 Wall 0003,             !- Name
    Wall,                     !- Surface Type
    ,                         !- Construction Name
    Thermal Zone 2,           !- Zone Name
    Outdoors,                 !- Outside Boundary Condition
    ,                         !- Outside Boundary Condition Object
    SunExposed,               !- Sun Exposure
    WindExposed,              !- Wind Exposure
    ,                         !- View Factor to Ground
    ,                         !- Number of Vertices
    0.0,                      !- Vertex 1 Xcoordinate
    2.9,                      !- Vertex 1 Ycoordinate
    1.458,                   !- Vertex 1 Zcoordinate
    0.0,                      !- Vertex 2 Xcoordinate
    2.9,                      !- Vertex 2 Ycoordinate
    0.7279,                      !- Vertex 2 Zcoordinate
    -2.14,                    !- Vertex 3 Xcoordinate
    2.9,                      !- Vertex 3 Ycoordinate
    0.7279,                      !- Vertex 3 Zcoordinate
    -2.14,                    !- Vertex 4 Xcoordinate
    2.9,                      !- Vertex 4 Ycoordinate
    1.458;                   !- Vertex 4 Zcoordinate

BuildingSurface:Detailed, 
    z2 Wall 0004,             !- Name
    Wall,                     !- Surface Type
    ,                         !- Construction Name
    Thermal Zone 2,           !- Zone Name
    Outdoors,                 !- Outside Boundary Condition
    ,                         !- Outside Boundary Condition Object
    SunExposed,               !- Sun Exposure
    WindExposed,              !- Wind Exposure
    ,                         !- View Factor to Ground
    ,                         !- Number of Vertices
    0.0,                      !- Vertex 1 Xcoordinate
    0.0,                      !- Vertex 1 Ycoordinate
    1.458,                   !- Vertex 1 Zcoordinate
    0.0,                      !- Vertex 2 Xcoordinate
    0.0,                      !- Vertex 2 Ycoordinate
    0.7279,                      !- Vertex 2 Zcoordinate
    0.0,                      !- Vertex 3 Xcoordinate
    2.9,                      !- Vertex 3 Ycoordinate
    0.7279,                      !- Vertex 3 Zcoordinate
    0.0,                      !- Vertex 4 Xcoordinate
    2.9,                      !- Vertex 4 Ycoordinate
    1.458;                   !- Vertex 4 Zcoordinate

BuildingSurface:Detailed, 
    z2 Roof 0001,             !- Name
    Roof,                     !- Surface Type
    ,                         !- Construction Name
    Thermal Zone 2,           !- Zone Name
    Outdoors,                 !- Outside Boundary Condition
    ,                         !- Outside Boundary Condition Object
    SunExposed,               !- Sun Exposure
    WindExposed,              !- Wind Exposure
    ,                         !- View Factor to Ground
    ,                         !- Number of Vertices
    0.0,                      !- Vertex 1 Xcoordinate
    0.0,                      !- Vertex 1 Ycoordinate
    1.458,                   !- Vertex 1 Zcoordinate
    0.0,                      !- Vertex 2 Xcoordinate
    2.9,                      !- Vertex 2 Ycoordinate
    1.458,                   !- Vertex 2 Zcoordinate
    -2.14,                    !- Vertex 3 Xcoordinate
    2.9,                      !- Vertex 3 Ycoordinate
    1.458,                   !- Vertex 3 Zcoordinate
    -2.14,                    !- Vertex 4 Xcoordinate
    0.0,                      !- Vertex 4 Ycoordinate
    1.458;                   !- Vertex 4 Zcoordinate

"""
idf = IDF()
idf.initreadtxt(idftxt)

idf.outputtype = 'compressed'
idf.printidf()


# Copyright (c) 2012 Santosh Philip
# =======================================================================
#  Distributed under the MIT License.
#  (See accompanying file LICENSE or copy at
#  http://opensource.org/licenses/MIT)
# =======================================================================
"""script to test idf reads.
Use this to test all the files in the example folder when a new version is released"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import argparse

import sys
# pathnameto_eppy = 'c:/eppy'
pathnameto_eppy = '../../'
sys.path.append(pathnameto_eppy)

import os
from eppy.modeleditor import IDF
import eppy.simpleread as simpleread

# iddfile = '/Applications/EnergyPlus-8-1-0/Energy+.idd'
# folder = '/Applications/EnergyPlus-8-1-0/ExampleFiles'
# python eppyreadtest_folder.py '/Applications/EnergyPlus-8-1-0/Energy+.idd'
# '/Applications/EnergyPlus-8-1-0/ExampleFiles'

def doreadtest(iddfile, folder, silent=False):
    """print out all the readtest results"""
    lst = os.listdir(folder)
    lst = [l for l in lst if l.endswith('.idf')]


    iddhandle = open(iddfile, 'r')
    for i, fname in enumerate(lst[355:359]): # lst[6:7]
                                    # if you want to test a specific file
        fname1 = "%s/%s" % (folder, fname)
        idfhandle1 = open(fname1, 'rb')
        idfhandle2 = open(fname1, 'rb')
        verbose = not silent
        result = simpleread.idfreadtest(
            iddhandle, idfhandle1, idfhandle2,
            verbose=verbose, save=False)
        print(i, result, fname)
        if result == False and (not silent):
            print("full filepath of file that failed the read test ->")
            print("    %s" % (fname1, ))
            print()
        idfhandle1.close()
        idfhandle2.close()
    iddhandle.close()

if __name__ == '__main__':
    # do the argparse stuff
    parser = argparse.ArgumentParser()
    parser.add_argument(
        'idd', action='store',
        help='location of idd file. ./somewhere/eplusv8-0-1.idd')
    parser.add_argument(
        'folder', action='store',
        help='location of folder with idf files. ./somewhere/examples')
    parser.add_argument(
        '--silent', action='store_true', default=False,
        help="don't print the first mismatching line")
    nspace = parser.parse_args()
    folder = nspace.folder
    iddfile = nspace.idd
    silent = nspace.silent
    doreadtest(iddfile, folder, silent)



# Copyright (c) 2012 Santosh Philip
# =======================================================================
#  Distributed under the MIT License.
#  (See accompanying file LICENSE or copy at
#  http://opensource.org/licenses/MIT)
# =======================================================================

"""just read the idf file"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import eppy.EPlusInterfaceFunctions.parse_idd as parse_idd
import eppy.EPlusInterfaceFunctions.eplusdata as eplusdata
import eppy.EPlusInterfaceFunctions.iddgroups as iddgroups

# from EPlusInterfaceFunctions import parse_idd
# from EPlusInterfaceFunctions import eplusdata

def readidf(idfname):
    """read the idf file"""
    # iddfile = sys.path[0] + '/EplusCode/Energy+.idd'
    iddfile = 'Energy+.idd'
    # iddfile = './EPlusInterfaceFunctions/E1.idd' # TODO : can the path name be not hard coded
    iddtxt = open(iddfile, 'r').read()
    block, commlst, commdct = parse_idd.extractidddata(iddfile)

    theidd = eplusdata.Idd(block, 2)
    data = eplusdata.Eplusdata(theidd, idfname)
    return data


def readiddidf(idfname):
    """read the idf file"""
    # iddfile = sys.path[0] + '/EplusCode/Energy+.idd'
    iddfile = 'Energy+.idd'
    # iddfile = './EPlusInterfaceFunctions/E1.idd' # TODO : can the path name be not hard coded
    iddtxt = open(iddfile, 'r').read()
    block, commlst, commdct = parse_idd.extractidddata(iddfile)

    theidd = eplusdata.Idd(block, 2)
    data = eplusdata.Eplusdata(theidd, idfname)
    return theidd, data

def readiddstuff(idfname):
    """read the idf file"""
    # iddfile = sys.path[0] + '/EplusCode/Energy+.idd'
    iddfile = 'Energy+.idd'
    # iddfile = './EPlusInterfaceFunctions/E1.idd' # TODO : can the path name be not hard coded
    iddtxt = open(iddfile, 'r').read()
    block, commlst, commdct = parse_idd.extractidddata(iddfile)

    theidd = eplusdata.Idd(block, 2)
    data = eplusdata.Eplusdata(theidd, idfname)
    return block, commlst, commdct



def readdatacommlst(idfname):
    """read the idf file"""
    # iddfile = sys.path[0] + '/EplusCode/Energy+.idd'
    iddfile = 'Energy+.idd'
    # iddfile = './EPlusInterfaceFunctions/E1.idd' # TODO : can the path name be not hard coded
    iddtxt = open(iddfile, 'r').read()
    block, commlst, commdct = parse_idd.extractidddata(iddfile)

    theidd = eplusdata.Idd(block, 2)
    data = eplusdata.Eplusdata(theidd, idfname)
    return data, commlst

def readdatacommdct(idfname, iddfile='Energy+.idd', commdct=None):
    """read the idf file"""
    if not commdct:
        block, commlst, commdct, idd_index = parse_idd.extractidddata(iddfile)
        theidd = eplusdata.Idd(block, 2)
    else:
        theidd = iddfile
    data = eplusdata.Eplusdata(theidd, idfname)
    return data, commdct, idd_index

def readdatacommdct1(
        idfname, iddfile='Energy+.idd',
        commdct=None, block=None):
    """read the idf file"""
    if not commdct:
        block, commlst, commdct, idd_index = parse_idd.extractidddata(iddfile)
        theidd = eplusdata.Idd(block, 2)
    else:
        theidd = eplusdata.Idd(block, 2)
        idd_index = {} # it should not get here :-(
    data = eplusdata.Eplusdata(theidd, idfname)
    return block, data, commdct, idd_index



"""Legacy code from EPlusInterface"""
# Copyright (C) 2004 Santosh Philip
# =======================================================================
#  Distributed under the MIT License.
#  (See accompanying file LICENSE or copy at
#  http://opensource.org/licenses/MIT)
# =======================================================================


# this is a test version ... not for real use
# dammit i am using it
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import copy
from six import StringIO
from six import string_types as str

import eppy.EPlusInterfaceFunctions.mylib2 as mylib2


def removecomment(astr, cphrase):
    """
    the comment is similar to that in python.
    any charachter after the # is treated as a comment
    until the end of the line
    astr is the string to be de-commented
    cphrase is the comment phrase"""
    # linesep = mylib3.getlinesep(astr)
    alist = astr.splitlines()
    for i in range(len(alist)):
        alist1 = alist[i].split(cphrase)
        alist[i] = alist1[0]

    # return string.join(alist, linesep)
    return '\n'.join(alist)


class Idd(object):

    """Idd object"""

    def __init__(self, dictfile, version=2):
        if version == 2:
            # version == 2. This is a just a flag I am using
            # it may wind up being the only type... then I can clean this up
            # and not use the other option
            self.dt, self.dtls = self.initdict2(dictfile)
            return
        self.dt, self.dtls = self.initdict(dictfile)

    def initdict2(self, dictfile):
        """initdict2"""
        dt = {}
        dtls = []
        adict = dictfile
        for element in adict:
            dt[element[0].upper()] = []  # dict keys for objects always in caps
            dtls.append(element[0].upper())
        return dt, dtls

    def initdict(self, fname):
        """initdict"""
        astr = mylib2.readfile(fname)
        nocom = removecomment(astr, '!')
        idfst = nocom
        alist = idfst.split(';')
        lss = []
        for element in alist:
            lst = element.split(',')
            lss.append(lst)

        for i in range(0, len(lss)):
            for j in range(0, len(lss[i])):
                lss[i][j] = lss[i][j].strip()

        dt = {}
        dtls = []
        for element in lss:
            if element[0] == '':
                continue
            dt[element[0].upper()] = []
            dtls.append(element[0].upper())

        return dt, dtls


class Eplusdata(object):

    """Eplusdata"""

    def __init__(self, dictfile=None, fname=None):
        # import pdb; pdb.set_trace()
        if fname == None and dictfile == None:
            self.dt, self.dtls = {}, []
        if isinstance(dictfile, str) and fname == None:
            self.initdict(dictfile)
        if isinstance(dictfile, Idd) and fname == None:
            self.initdict(dictfile)
        if isinstance(fname, str) and isinstance(dictfile, str):
            fnamefobject = open(fname, 'rb')
            self.makedict(dictfile, fnamefobject)
        if isinstance(fname, str) and isinstance(dictfile, Idd):
            fnamefobject = open(fname, 'rb')
            self.makedict(dictfile, fnamefobject)
        try:
            # will fail in python3 because of file
            if (isinstance(fname, (file, StringIO)) and
                    isinstance(dictfile, str)):
                self.makedict(dictfile, fname)
            if (isinstance(fname, (file, StringIO)) and
                    isinstance(dictfile, Idd)):
                self.makedict(dictfile, fname)
        except NameError:
            from io import IOBase
            if (isinstance(fname, (IOBase, StringIO)) and
                    isinstance(dictfile, str)):
                self.makedict(dictfile, fname)
            if (isinstance(fname, (IOBase, StringIO)) and
                    isinstance(dictfile, Idd)):
                self.makedict(dictfile, fname)

    def __repr__(self):
        # print dictionary
        dt = self.dt
        dtls = self.dtls
        UNIXSEP = "\n"
        DOSSEP = UNIXSEP # using a unix EOL
        astr = ''
        for node in dtls:
            nodedata = dt[node.upper()]
            for block in nodedata:
                for i in range(len(block)):
                    fformat = '     %s,' + DOSSEP
                    if i == 0:
                        fformat = '%s,' + DOSSEP
                    if i == len(block) - 1:
                        fformat = '     %s;' + DOSSEP * 2
                    astr = astr + fformat % block[i]

        return astr

    #------------------------------------------
    def initdict(self, fname):
        """create a blank dictionary"""
        if isinstance(fname, Idd):
            self.dt, self.dtls = fname.dt, fname.dtls
            return self.dt, self.dtls

        astr = mylib2.readfile(fname)
        nocom = removecomment(astr, '!')
        idfst = nocom
        alist = idfst.split(';')
        lss = []
        for element in alist:
            lst = element.split(',')
            lss.append(lst)

        for i in range(0, len(lss)):
            for j in range(0, len(lss[i])):
                lss[i][j] = lss[i][j].strip()

        dt = {}
        dtls = []
        for element in lss:
            if element[0] == '':
                continue
            dt[element[0].upper()] = []
            dtls.append(element[0].upper())

        self.dt, self.dtls = dt, dtls
        return dt, dtls

    #------------------------------------------
    def makedict(self, dictfile, fnamefobject):
        """stuff file data into the blank dictionary"""
        #fname = './exapmlefiles/5ZoneDD.idf'
        #fname = './1ZoneUncontrolled.idf'
        if isinstance(dictfile, Idd):
            localidd = copy.deepcopy(dictfile)
            dt, dtls = localidd.dt, localidd.dtls
        else:
            dt, dtls = self.initdict(dictfile)
        # astr = mylib2.readfile(fname)
        astr = fnamefobject.read()
        try:
            astr = astr.decode('ISO-8859-2')
        except AttributeError:
            pass
        fnamefobject.close()
        nocom = removecomment(astr, '!')
        idfst = nocom
        # alist = string.split(idfst, ';')
        alist = idfst.split(';')
        lss = []
        for element in alist:
            # lst = string.split(element, ',')
            lst = element.split(',')
            lss.append(lst)

        for i in range(0, len(lss)):
            for j in range(0, len(lss[i])):
                lss[i][j] = lss[i][j].strip()

        for element in lss:
            node = element[0].upper()
            if node in dt:
                # stuff data in this key
                dt[node.upper()].append(element)
            else:
                # scream
                if node == '':
                    continue
                print('this node -%s-is not present in base dictionary' %
                      (node))

        self.dt, self.dtls = dt, dtls
        return dt, dtls

    def replacenode(self, othereplus, node):
        """replace the node here with the node from othereplus"""
        node = node.upper()
        self.dt[node.upper()] = othereplus.dt[node.upper()]

    def add2node(self, othereplus, node):
        """add the node here with the node from othereplus
        this will potentially have duplicates"""
        node = node.upper()
        self.dt[node.upper()] = self.dt[node.upper()] + \
            othereplus.dt[node.upper()]

    def addinnode(self, otherplus, node, objectname):
        """add an item to the node.
        example: add a new zone to the element 'ZONE' """
        # do a test for unique object here
        newelement = otherplus.dt[node.upper()]

    def getrefs(self, reflist):
        """
        reflist is got from getobjectref in parse_idd.py
        getobjectref returns a dictionary.
        reflist is an item in the dictionary
        getrefs gathers all the fields refered by reflist
        """
        alist = []
        for element in reflist:
            if element[0].upper() in self.dt:
                for elm in self.dt[element[0].upper()]:
                    alist.append(elm[element[1]])
        return alist


#------------------------------------------



# Copyright (c) 2012 Tuan Tran
# =======================================================================
#  Distributed under the MIT License.
#  (See accompanying file LICENSE or copy at
#  http://opensource.org/licenses/MIT)
# =======================================================================

""""This module is used for calculation of zone area for E+ surfaces"""
# Wrote by Tuan Tran trantuan@hawaii.edu / tranhuuanhtuan@gmail.com
# School of Architecture, University of Hawaii at Manoa
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

# import area_surface as surf
import eppy.geometry.surface as surface

def area(poly):
    """Calculation of zone area"""
    poly_xy = []
    num = len(poly)
    for i in range(num):
        poly[i] = poly[i][0:2] + (0,)
        poly_xy.append(poly[i])
    return surface.area(poly)



# Copyright (c) 2012 Santosh Philip
# =======================================================================
#  Distributed under the MIT License.
#  (See accompanying file LICENSE or copy at
#  http://opensource.org/licenses/MIT)
# =======================================================================

"""use epbunch"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from eppy.EPlusInterfaceFunctions import readidf
import eppy.bunchhelpers as bunchhelpers
from eppy.bunch_subclass import EpBunch
# from eppy.bunch_subclass import fieldnames, fieldvalues
import eppy.iddgaps as iddgaps
import eppy.function_helpers as fh
from eppy.idf_msequence import Idf_MSequence


def iddversiontuple(afile):
    """given the idd file or filehandle, return the version handle"""
    def versiontuple(vers):
        """version tuple"""
        return tuple([int(num) for num in vers.split(".")])
    try:
        fhandle = open(afile, 'rb')
    except TypeError:
        fhandle = afile
    line1 = fhandle.readline()
    try:
        line1 = line1.decode('ISO-8859-2')
    except AttributeError:
        pass
    line = line1.strip()
    if line1 == '':
        return (0,)
    vers = line.split()[-1]
    return versiontuple(vers)


def makeabunch(commdct, obj, obj_i):
    """make a bunch from the object"""
    objidd = commdct[obj_i]
    objfields = [comm.get('field') for comm in commdct[obj_i]]
    objfields[0] = ['key']
    objfields = [field[0] for field in objfields]
    obj_fields = [bunchhelpers.makefieldname(field) for field in objfields]
    bobj = EpBunch(obj, obj_fields, objidd)
    return bobj


def makebunches(data, commdct):
    """make bunches with data"""
    bunchdt = {}
    ddtt, dtls = data.dt, data.dtls
    for obj_i, key in enumerate(dtls):
        key = key.upper()
        bunchdt[key] = []
        objs = ddtt[key]
        for obj in objs:
            # if obj[0] == "Construction:WindowDataFile":
            #     print obj
            bobj = makeabunch(commdct, obj, obj_i)
            bunchdt[key].append(bobj)
    return bunchdt


def makebunches_alter(data, commdct, theidf):
    """make bunches with data"""
    bunchdt = {}
    dt, dtls = data.dt, data.dtls
    for obj_i, key in enumerate(dtls):
        key = key.upper()
        objs = dt[key]
        list1 = []
        for obj in objs:
            bobj = makeabunch(commdct, obj, obj_i)
            list1.append(bobj)
        bunchdt[key] = Idf_MSequence(list1, objs, theidf)
        # print "id(objs)", id(objs)
        # print "id(dt[key])", id(dt[key])
        # print "id(bunchdt[key].list2)", id(bunchdt[key].list2)
    return bunchdt


def convertfields(key_comm, obj):
    """convert the float and interger fields"""
    def apass(aaa):
        """pass thru"""
        return aaa
    typefunc = dict(integer=int, real=float)
    # types = [comm.get('type', [None])[0] for comm in key_comm]
    types = []
    for comm in key_comm:
        types.append(comm.get('type', [None])[0])
    convs = [typefunc.get(typ, apass) for typ in types]
    for i, (val, conv) in enumerate(zip(obj, convs)):
        try:
            val = conv(val)
            obj[i] = val
        except ValueError:
            pass
    return obj


def convertallfields(data, commdct):
    """docstring for convertallfields"""
    # import pdbdb; pdb.set_trace()
    for key in list(data.dt.keys()):
        objs = data.dt[key]
        for i, obj in enumerate(objs):
            key_i = data.dtls.index(key)
            key_comm = commdct[key_i]
            obj = convertfields(key_comm, obj)
            objs[i] = obj


def addfunctions(dtls, bunchdt):
    """add functions to the objects"""
    snames = [
        "BuildingSurface:Detailed",
        "Wall:Detailed",
        "RoofCeiling:Detailed",
        "Floor:Detailed",
        "FenestrationSurface:Detailed",
        "Shading:Site:Detailed",
        "Shading:Building:Detailed",
        "Shading:Zone:Detailed", ]
    for sname in snames:
        if sname.upper() in bunchdt:
            surfaces = bunchdt[sname.upper()]
            for surface in surfaces:
                func_dict = {
                    'area': fh.area,
                    'height': fh.height,  # not working correctly
                    'width': fh.width,  # not working correctly
                    'azimuth': fh.azimuth,
                    'tilt': fh.tilt,
                    'coords': fh.getcoords,  # needed for debugging
                }
                try:
                    surface.__functions.update(func_dict)
                except KeyError as e:
                    surface.__functions = func_dict
    # add common functions
    # for name in dtls:
    #     for idfobject in bunchdt[name]:
    # idfobject.__functions
    #         idfobject['__functions']['fieldnames'] = fieldnames
    #         idfobject['__functions']['fieldvalues'] = fieldvalues
    #         idfobject['__functions']['getrange'] = GetRange(idfobject)
    #         idfobject['__functions']['checkrange'] = CheckRange(idfobject)

def addfunctions2new(abunch, key):
    """add functions to a new bunch/munch object"""
    snames = [
        "BuildingSurface:Detailed",
        "Wall:Detailed",
        "RoofCeiling:Detailed",
        "Floor:Detailed",
        "FenestrationSurface:Detailed",
        "Shading:Site:Detailed",
        "Shading:Building:Detailed",
        "Shading:Zone:Detailed", ]
    snames = [sname.upper() for sname in snames]
    if key in snames:
        func_dict = {
            'area': fh.area,
            'height': fh.height,  # not working correctly
            'width': fh.width,  # not working correctly
            'azimuth': fh.azimuth,
            'tilt': fh.tilt,
            'coords': fh.getcoords,  # needed for debugging
        }
        try:
            abunch.__functions.update(func_dict)
        except KeyError as e:
            abunch.__functions = func_dict
    return abunch


def idfreader(fname, iddfile, conv=True):
    """read idf file and return bunches"""
    data, commdct, idd_index = readidf.readdatacommdct(fname, iddfile=iddfile)
    if conv:
        convertallfields(data, commdct)
    # fill gaps in idd
    ddtt, dtls = data.dt, data.dtls
    # skiplist = ["TABLE:MULTIVARIABLELOOKUP"]
    nofirstfields = iddgaps.missingkeys_standard(
        commdct, dtls,
        skiplist=["TABLE:MULTIVARIABLELOOKUP"])
    iddgaps.missingkeys_nonstandard(None, commdct, dtls, nofirstfields)
    bunchdt = makebunches(data, commdct)
    return bunchdt, data, commdct, idd_index


def idfreader1(fname, iddfile, theidf, conv=True, commdct=None, block=None):
    """read idf file and return bunches"""
    versiontuple = iddversiontuple(iddfile)
    # import pdb; pdb.set_trace()
    block, data, commdct, idd_index = readidf.readdatacommdct1(
        fname,
        iddfile=iddfile,
        commdct=commdct,
        block=block)
    if conv:
        convertallfields(data, commdct)
    # fill gaps in idd
    ddtt, dtls = data.dt, data.dtls
    if versiontuple < (8,):
        skiplist = ["TABLE:MULTIVARIABLELOOKUP"]
    else:
        skiplist = None
    nofirstfields = iddgaps.missingkeys_standard(
        commdct, dtls,
        skiplist=skiplist)
    iddgaps.missingkeys_nonstandard(block, commdct, dtls, nofirstfields)
    # bunchdt = makebunches(data, commdct)
    bunchdt = makebunches_alter(data, commdct, theidf)
    return bunchdt, block, data, commdct, idd_index, versiontuple



# Copyright (c) 2012 Santosh Philip
# =======================================================================
#  Distributed under the MIT License.
#  (See accompanying file LICENSE or copy at
#  http://opensource.org/licenses/MIT)
# =======================================================================
"""
script to test idf reads in eppy.
Use to read test a single idf file
This may may be one of the files that failed when running eppyreadtest_folder.py
-
eppyreadtest_file.py will output two files:
1. simpleread.idf
2. eppyread.idf

The idf objects in both are sorted in ascending order. The files should be
identical. If there is a mismatch, eppy made a mistake in the read. Comparing
the two files will show you where the error occurred.
"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

import argparse

import sys
# pathnameto_eppy = 'c:/eppy'
pathnameto_eppy = '../../'
sys.path.append(pathnameto_eppy)

import os
from eppy.modeleditor import IDF
import eppy.simpleread as simpleread

# iddfile = '/Applications/EnergyPlus-8-1-0/Energy+.idd'
# folder = '/Applications/EnergyPlus-8-1-0/ExampleFiles'
# python eppyreadtest_file.py '/Applications/EnergyPlus-8-1-0/Energy+.idd'
# '/Applications/EnergyPlus-8-1-0/ExampleFiles/RetailPackagedTESCoil.idf'
# for non-failure -> SeriesActiveBranch.idf
# python eppyreadtest_file.py '/Applications/EnergyPlus-8-1-0/Energy+.idd'
# '/Applications/EnergyPlus-8-1-0/ExampleFiles/SeriesActiveBranch.idf'


def doreadtest(iddfile, folder, silent=False):
    """print out all the readtest results"""


    iddhandle = open(iddfile, 'r')
    fname1 = thefile
    idfhandle1 = open(fname1, 'rb')
    idfhandle2 = open(fname1, 'rb')
    verbose = not silent
    result = simpleread.idfreadtest(
        iddhandle, idfhandle1, idfhandle2,
        verbose=verbose, save=False)
    # print result,   fname
    if result == False and (not silent):
        print("first mismatch at the above line numbers")
        print("full filepath of file that failed the read test ->")
        print("    %s" % (fname1, ))
        print()
        print("compare files 'simpleread.idf' vs 'eppyread.idf'")
        print()
    else:
        print("file passed test")
    idfhandle1.close()
    idfhandle2.close()
    iddhandle.close()

if __name__ == '__main__':
    # do the argparse stuff
    parser = argparse.ArgumentParser(__doc__)
    parser.add_argument(
        'idd', action='store',
        help='location of idd file. ./somewhere/eplusv8-0-1.idd')
    parser.add_argument(
        'thefile', action='store',
        help='location the idf file. ./somewhere/examples/thisfile.idf')
    nspace = parser.parse_args()
    thefile = nspace.thefile
    iddfile = nspace.idd
    doreadtest(iddfile, thefile)



# Copyright (c) 2012 Santosh Philip
# =======================================================================
#  Distributed under the MIT License.
#  (See accompanying file LICENSE or copy at
#  http://opensource.org/licenses/MIT)
# =======================================================================

"""some init routines to do the documentation"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from IPython.display import Image
from IPython.display import display_pretty, display_html, display_jpeg, display_png, display_json, display_latex, display_svg
idfeditor = Image(filename='../docs/images/idf3.png')
material_lists = Image(filename='../docs/images/material_lists.png')
html_snippet1 = Image(filename='../docs/images/html1.png')
html_snippet2 = Image(filename='../docs/images/html2.png')
plantloop1 = Image(filename='../docs/images/hhh1.png')
plantloop2 = Image(filename='../docs/images/hhh_new.png')



"""
Subclass from collections.MutableSequence to get finer control over a list like
object.

This is to work with issue 40 in github:

idf1.idfobjects['BUILDING'] is a list and is not connected to
idf1.model.dt['BUILDING']

List has to be subclassed to solve this problem.

# Alex Martelli describes how to use collections.MutableSequence in
# <http://stackoverflow.com/questions/3487434/overriding-append-method-after-inheriting-from-a-python-list>

"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals


import collections

from eppy.bunch_subclass import EpBunch


class Idf_MSequence(collections.MutableSequence):
    """Used to keep IDF.idfobjects in sync with IDF.model.dt."""
    def __init__(self, list1, list2, theidf):
        """Initialise the object.

        Parameters
        ----------
        list1 : list
            Bunches (IDF.idfobjects).
        list2 : list
            Objects (IDF.model.dt).
        theidf : modeleditor.IDF
            The IDF.

        """
        super(Idf_MSequence, self).__init__()
        self.list1 = list1
        self.list2 = list2
        self.theidf = theidf
        for v in self.list1:
            if isinstance(v, EpBunch):
                v.theidf = self.theidf

    def __getitem__(self, i):
        """Gets an idfobject (bunch) from list1."""
        return self.list1[i]

    def __setitem__(self, i, v):
        """Sets an idfobject (bunch) to list1 and its object to list2."""
        self.list1[i] = v
        self.list2[i] = v.obj

    def __delitem__(self, i):
        """Deletes an idfobject (bunch) from list1 and its object from list2."""
        v = self.list1[i]
        if isinstance(v, EpBunch):
            v.theidf = None
        del self.list1[i]
        del self.list2[i]

    def __len__(self):
        """Number of idfobjects (bunches)."""
        return len(self.list1)

    def insert(self, i, v):
        """Insert an idfobject (bunch) to list1 and its object to list2."""
        self.list1.insert(i, v)
        self.list2.insert(i, v.obj)
        if isinstance(v, EpBunch):
            v.theidf = self.theidf

    def __str__(self):
        """String representation of the list of idfobjects (bunches)."""
        return str(self.list1)

    def __repr__(self):
        """Repr representation of the list of idfobjects (bunches)."""
        return str(self.list1)

    def __eq__(self, other):
        """Test for equality uses the IDF.model.dt list, list2."""
        return self.list2 == other.list2
